(*---------------------------------------------------------------------------
  :Program.    WildDemo1.mod
  :Author.     Fridtjof Siebert
  :Address.    Nobileweg 67, D-7-Stgt-40
  :Shortcut.   [fbs]
  :Version.    1.0
  :Date.       14-Jan-88
  :Copyright.  PD
  :Language.   Modula-II
  :Translator. M2Amiga v3.1d
  :Imports.    ARP
  :History.    Original C Version by Scott Ballantyne
  :Contents.   Demonstration of how to use the arp.library wildcard routines.
  :Contents..  This program simulates a sort of dir command
---------------------------------------------------------------------------*)

MODULE WildDemo1;

FROM SYSTEM   IMPORT ADR, LONGSET;
FROM Arts     IMPORT dosCmdBuf, dosCmdLen, wbStarted, Terminate, detectCtrlC,
                     returnVal;
FROM ARP      IMPORT StdAnchorPathPtr, errorBreak, errorBufferOverflow, GADS,
                     ArpAlloc, FindFirst, FindNext, FreeAnchorChain,
                     AnchorPathPtr;
FROM Dos      IMPORT ctrlC, ctrlD, objectNotFound, noMoreEntries;
FROM ASCII    IMPORT ht;
FROM Terminal IMPORT WriteString, WriteLn, Write;

CONST
  cliTemplate = "Pattern";
  cliHelp = "List Files or directories by wildcards";

TYPE
  String = ARRAY[0..255] OF CHAR;

VAR
  argc: INTEGER;
  argv: POINTER TO String;
  Anchor: StdAnchorPathPtr;
  Pattern: String;
  Result: LONGINT;

(*------  MAIN:  ------*)

BEGIN
  detectCtrlC := FALSE; (* Just to demonstrate break *)

  IF wbStarted THEN
    argc := 0;
  ELSE
    argc := GADS(dosCmdBuf,dosCmdLen,ADR(cliHelp),ADR(argv),ADR(cliTemplate));
  END;

  IF argc<1 THEN
    Pattern := "*";      (* All files/dirs in current directory *)
  ELSE
    Pattern := argv^;
  END;

  Anchor := ArpAlloc(SIZE(Anchor^));
  IF Anchor=NIL THEN
    WriteString("No memory!"); WriteLn;
    Terminate(0);
  END;

  Anchor^.anchor.length := 255;  (* Want full path built *)

  (* We will be breaking on control 'C's
     and on control 'D's, just to show how *)
  Anchor^.anchor.breakBits := LONGSET{ctrlC,ctrlD};

  Result := FindFirst(ADR(Pattern),AnchorPathPtr(Anchor));

  WHILE Result=0 DO
    WriteString(Anchor^.buffer);     (* Display name *)
    IF Anchor^.anchor.info.dirEntryType >= 0 THEN
      Write(ht); WriteString("(dir)");        (* Display type *)
    END;
    WriteLn;
    Result := FindNext(AnchorPathPtr(Anchor));
  END;

  (* Free the Anchor chain built by the above functions *)
  FreeAnchorChain(AnchorPathPtr(Anchor));

  (* Finally, check the result - if it is ERROR_NO_MORE_ENTRIES then
   * we are exiting normally, otherwise, it is either a ^C break
   * or an actual error of some kind.
   *)

  CASE Result OF
  errorBreak:              (* Control 'C' *)
    IF ctrlC IN Anchor^.anchor.foundBreak THEN
      WriteString("***Break on '^C'");
    ELSE
      WriteString("***Break on '^D'");
    END; |
  objectNotFound:
    WriteString("Can't find "); WriteString(Pattern);
    returnVal := 20; |
  errorBufferOverflow:
    WriteString("I should have allocated a larger buffer. Sorry!");
    returnVal := 20; |
  noMoreEntries: |
  ELSE
    WriteString("An IO error occured!");
  END;
  WriteLn;
END WildDemo1.
