(*---------------------------------------------------------------------------
  :Program.    WildDemo2.mod
  :Author.     Fridtjof Siebert
  :Address.    Nobileweg 67, D-7-Stgt-40
  :Shortcut.   [fbs]
  :Version.    1.0
  :Date.       15-Jan-88
  :Copyright.  PD
  :Language.   Modula-II
  :Translator. M2Amiga v3.1d
  :Imports.    ARP
  :History.    Original C Version by Scott Ballantyne
  :Contents.   Search filesystem for patterns, and separete into directories
  :Contents.   and files, sorting each separately usig DA lists
---------------------------------------------------------------------------*)

MODULE WildDemo2;

FROM SYSTEM  IMPORT ADR, LONGSET;
FROM Arts    IMPORT dosCmdBuf, dosCmdLen, Terminate, wbStarted, returnVal;
FROM ARP     IMPORT AnchorPathPtr, DirectoryEntryPtr, AnchorPtr, GADS,
                    FindFirst, FindNext, ArpAlloc, AddDANode, errorBreak,
                    errorBufferOverflow, FreeDAList, FreeAnchorChain,
                    DirEntryTypes, DirEntryTypeSet;
FROM Dos     IMPORT ctrlC, objectNotFound, noMoreEntries;
FROM ASCII   IMPORT ht, lf;
FROM InOut   IMPORT WriteString, WriteLn, WriteInt, Write;
FROM Strings IMPORT Length;

CONST
  cliTemplate = "Pattern";
  cliHelp = "List Files or Directories by wildcard";

  directory = DirEntryTypeSet{};         (* type of entry, used in DAList *)
  file      = DirEntryTypeSet{devices};

TYPE
  String = ARRAY[0..255] OF CHAR;

VAR
  argc: INTEGER;               (* # of arguments                      *)
  argv: POINTER TO String;     (* argument from GADS()                *)
  Pattern: String;             (* file pattern                        *)
  FileList: DirectoryEntryPtr; (* list of files and dirs              *)
  Anchor: AnchorPathPtr;       (* for Patternsearch                   *)
  de: DirectoryEntryPtr;       (* for searching list                  *)
  fcount,dcount: LONGINT;      (* count files and directories         *)
  key: DirEntryTypeSet;        (* type of entry (file/dir, see above) *)
  Result: LONGINT;             (* Dos-Error #                         *)
  lflag: BOOLEAN;              (* flag for Columnize()                *)

(* Simple routine to print a file or directory name in 2 columns *)

PROCEDURE Columnize(name: ARRAY OF CHAR);

VAR s: String;

BEGIN
  IF lflag THEN Write(lf); Write(ht) END;
  WriteString(name);
  IF lflag AND (Length(name)<32) THEN
    s := "                                ";
    s[32-Length(name)] := 0C;
    WriteString(s);
  END;
  lflag := NOT lflag;
END Columnize;

(*------  MAIN:  ------*)

BEGIN
  IF wbStarted THEN
    argc := 0;
  ELSE
    argc := GADS(dosCmdBuf,dosCmdLen,ADR(cliHelp),ADR(argv),ADR(cliTemplate));
    IF argc<0 THEN WriteString("bad args"); WriteLn; Terminate(0) END;
  END;

  IF argc<1 THEN
    Pattern := "*";      (* All files/dirs in current directory *)
  ELSE
    Pattern := argv^;
  END;

  (* Get our AnchorBase *)

  Anchor := ArpAlloc(SIZE(Anchor^));
  IF Anchor=NIL THEN
    WriteString("No memory!"); WriteLn;
    Terminate(0);
  END;

  Anchor^.length := 0;
  Anchor^.breakBits := LONGSET{ctrlC};

  Result := FindFirst(ADR(Pattern),Anchor);

  WHILE Result = 0 DO
    IF Anchor^.info.dirEntryType >= 0 THEN
      INC(dcount); key := directory;
    ELSE
      INC(fcount); key := file;
    END;
    IF AddDANode(ADR(Anchor^.info.fileName),ADR(FileList),0,key)=NIL THEN
      WriteString("Out of memory!");
      FreeDAList( FileList );
      Terminate(0);
    END;
    Result := FindNext(Anchor);
  END;

  (* Free the Anchor chain built by the above functions *)

  FreeAnchorChain(Anchor);

  CASE Result OF
  errorBreak: WriteString("***Break"); |
  objectNotFound:
    WriteString("Can't find "); WriteString(Pattern);
    returnVal := 20; |
  noMoreEntries: (* Now munge through the DA list, displaying the sorted files *)
    de := FileList;
    WriteInt(dcount,5); WriteString(" Directories:");
    IF de^.type=directory THEN    (* While we have directories *)
      lflag := TRUE;
      WHILE (de^.type=directory) AND (de#NIL) DO
        Columnize(de^.name);
        de := de^.next;
      END;
    END;
    WriteLn; WriteInt(fcount,5); WriteString(" Files:");
    IF de#NIL THEN
      lflag := TRUE;
      WHILE (de^.type=file) AND (de#NIL) DO
        Columnize(de^.name);
        de := de^.next;
      END;
    END; |
  ELSE
    WriteString("Error #"); WriteInt(Result,5);
    returnVal := 20;
  END;
  FreeDAList(FileList);
  WriteLn;
END WildDemo2.
