(*---------------------------------------------------------------------------
   :Program.    Kurve
   :Author.     Jrg Wesemann
   :Address.    Auf der Heide 10, 2807 Achim - Baden
   :Phone.      (0)4202 / 70658
   :Shortcut.   [jwe]
   :Version.    2.94
   :Date.       23.03.89
   :Copyright.  PD
   :Language.   Modula-II
   :Translator. M2Amiga
   :Imports.    none.
   :Update.     none.
   :Contents.   Stellt den Parser fr Lineare Gleichungssysteme
   :Remark.     none.
---------------------------------------------------------------------------*)


IMPLEMENTATION MODULE LGSparser;

FROM SYSTEM 		IMPORT 	ADR;
FROM Strings 		IMPORT 	Occurs,first,last,Length;
FROM MathLib0 		IMPORT 	sqrt,arctan,cos,exp,ln,sin;
FROM RealConversions 	IMPORT 	StrToReal;
FROM Arts		IMPORT  BreakPoint;


PROCEDURE LGSparse(argument: ARRAY OF CHAR;X:REAL;VAR bool: BOOLEAN):REAL;

  CONST
    CLS		= CHR(12);
    NULL	= "";
    TOKSIZE	= 20;

  VAR
    now 		: CARDINAL;
    arglen,lauf 	: INTEGER;
    exprs		: ARRAY [0..90]  OF CHAR;
    ans,Xres 		: REAL;  

PROCEDURE Round(n : REAL):REAL;

BEGIN
  IF n<0.0 THEN
    RETURN(REAL(TRUNC(n-0.5)));
  ELSE
    RETURN(REAL(TRUNC(n+0.5)));
  END;
END Round;


PROCEDURE Fak(n : REAL):REAL;

BEGIN
  IF n>20.0 THEN 
    BreakPoint(ADR("! Fak(x) -> Fak(20) !"));
    n:= 20.0;
  END;
  n:= REAL(TRUNC(n+0.5));
  IF (n=1.0) OR (n=0.0) THEN
    RETURN 1.0;
  ELSE
    RETURN (n*Fak(n-1.0));
  END;
END Fak;


PROCEDURE fastnull(zahl:REAL):BOOLEAN;

BEGIN
  RETURN (ABS(zahl)<0.000001);
END fastnull;

PROCEDURE binary(a,b:REAL; tok: ARRAY OF CHAR):REAL;
  
  VAR
    c		: REAL;
    index 	: INTEGER;

BEGIN
  CASE tok[0] OF
    | "+": RETURN (a+b)
    | "-": RETURN (a-b)
    | "*": RETURN (a*b)
    | "/": IF NOT fastnull(b) THEN
             RETURN (a/b);
           ELSE
             BreakPoint(ADR("Nenner durch 0.00001 ersetzt"));
             RETURN(a/0.00001);
           END;
    | "^": c:= a;
           FOR index:= 1 TO INTEGER(ABS(b))-1 DO
             a:= a*c;
           END;
           IF b<0.00001 THEN 
             IF NOT fastnull(a) THEN
               a:= 1.0/a;
             ELSE					(* IF 2		*)
               a:= 1.0/0.000001;
             END;					(* IF 2		*)
           END;						(* IF 1		*)
           IF ABS(b)<0.000001 THEN
             a:= 1.0;
           END;
           RETURN a        |
  ELSE
    bool:= FALSE;
    RETURN 1.0;
  END;							(* CASE 1	*)
END binary;


PROCEDURE stripleer;

BEGIN
  WHILE (exprs[now]=" ") DO
    INC(now);
  END;
END stripleer;



PROCEDURE getfn(VAR tok: ARRAY OF CHAR);

  VAR
    toklauf : CARDINAL;

BEGIN
  FOR toklauf:= 0 TO TOKSIZE DO
    tok[toklauf]:=" ";
  END;
  toklauf:=0;
  stripleer;
  WHILE ( (exprs[now]>="A") AND (exprs[now]<="Z") ) DO
     tok[toklauf]:=exprs[now];
     INC(toklauf);
     INC(now);
  END;
END getfn;



PROCEDURE getnumb(VAR tok: ARRAY OF CHAR);

  VAR
    toklauf: CARDINAL;
    
BEGIN
  FOR toklauf:= 0 TO TOKSIZE DO
    tok[toklauf]:=" ";
  END;
  stripleer;
  toklauf:=0;
  WHILE 
  (  ( (exprs[now]>="0") AND (exprs[now]<="9") ) OR (exprs[now]=".") ) DO
    tok[toklauf]:= exprs[now];
    INC(toklauf);
    INC(now);
  END;
END getnumb;      


PROCEDURE gettoken(VAR tok: ARRAY OF CHAR);

  VAR
    c: CHAR;

BEGIN
  stripleer;
  c:=exprs[now];
  IF (
      (c="+") OR (c="-") OR (c="*") OR (c="/") OR (c="^") OR (c=")") OR 
      (c="&")
     ) THEN
    tok[0]:= exprs[now];
    INC(now);
  ELSE
    tok[0]:= NULL;
  END;
END gettoken;
  
  
  
PROCEDURE parse0():REAL; FORWARD;


PROCEDURE parse5():REAL;

  VAR
    tok,zahlstr 	: ARRAY[0..TOKSIZE] OF CHAR;
    a 			: REAL;
    err			:BOOLEAN;
    toklen,index	: CARDINAL;
  
  PROCEDURE fneval():REAL;
  
  BEGIN
    IF tok[0]="X" THEN
      RETURN Xres;
    ELSIF (Occurs(tok,first,"PI",FALSE)#last) THEN
      RETURN 3.14159265;  
    ELSIF (Occurs(tok,first,"SIN",FALSE)#last) THEN
      RETURN (sin(parse5()));
    ELSIF (Occurs(tok,first,"SQRT",FALSE)#last) THEN
      RETURN (sqrt(parse5()));
    ELSIF (Occurs(tok,first,"EXP",FALSE)#last) THEN
      RETURN (exp(parse5()));
    ELSIF (Occurs(tok,first,"LN",FALSE)#last) THEN
      RETURN (ln(parse5()));
    ELSIF (Occurs(tok,first,"ATN",FALSE)#last) THEN
      RETURN (arctan(parse5()));
    ELSIF (Occurs(tok,first,"COS",FALSE)#last) THEN
      RETURN (cos(parse5()));
    ELSIF (Occurs(tok,first,"FAK",FALSE)#last) THEN
      RETURN (Fak(parse5()));
    ELSIF (Occurs(tok,first,"ABS",FALSE)#last) THEN
      RETURN (ABS(parse5()));
    ELSIF (Occurs(tok,first,"TRUNC",FALSE)#last) THEN
      RETURN (REAL(TRUNC(parse5())));
    ELSIF (Occurs(tok,first,"ROUND",FALSE)#last) THEN
      RETURN (Round(parse5()));
    ELSIF (Occurs(tok,first,"EE",FALSE)#last) THEN
      RETURN 2.7182818284;
    ELSE
      bool:= FALSE;
      RETURN 1.00;
    END;
  END fneval;

BEGIN							(* parse5 Main	*)
  getfn(tok);
  IF tok[0]#" " THEN
    RETURN (fneval());
  END;
  getnumb(tok);
  IF tok[0]#" " THEN
    StrToReal(tok,a,err);
    IF err THEN
      bool:= FALSE;
      RETURN 1.00;
    ELSE						(* IF 2		*)
      RETURN(a)
    END;						(* IF 2		*)
  END;							(* IF 1		*)
  IF (exprs[now]="(") THEN
    INC(now);
    a:= parse0();
    IF (exprs[now]=")") THEN
      INC(now);
    ELSE						(* IF 4		*)
      bool:= FALSE;
      RETURN(1.00);
    END;						(* IF 4		*)
    RETURN a;
  END;							(* IF 3		*)
  bool:= FALSE;
  RETURN 1.00;
END parse5;



PROCEDURE parse4():REAL;

  VAR
    tok 	: ARRAY [0..TOKSIZE] OF CHAR;

BEGIN
  gettoken(tok);
  IF (tok[0]="-") THEN
    RETURN (-(parse4()));
  ELSIF (tok[0]="+") THEN
    RETURN(parse4());
  ELSIF (tok[0]=NULL) THEN
    RETURN (parse5());
  END;
  DEC(now);
  RETURN(parse5());
END parse4;


PROCEDURE parse3():REAL;

  VAR
    tok		: ARRAY [0..TOKSIZE] OF CHAR;
    a 		: REAL;

BEGIN
  a:= parse4();
  WHILE (exprs[now]#NULL) DO
    gettoken(tok);
    IF (  (tok[0]="^") OR (tok[0]="&")  ) THEN
      a:= binary(a,parse4(),tok);
    ELSE
      DEC(now);
    END;
    RETURN a;
  END;
  RETURN a;
END parse3;



PROCEDURE parse2():REAL;

  VAR
    tok		: ARRAY [0..TOKSIZE] OF CHAR;
    a		: REAL;

BEGIN
  a:= parse3();
  WHILE (exprs[now]#NULL) DO
    gettoken(tok);
    IF ((tok[0]="*") OR (tok[0]="/")) THEN
      a:= binary(a,parse3(),tok);
    ELSE						(* IF 1		*)
      DEC(now);
      RETURN a;
    END;						(* IF 1		*)
  END;							(* WHILE 1	*)
  RETURN a;
END parse2;


PROCEDURE parse0():REAL;

  VAR
    tok 	: ARRAY [0..TOKSIZE] OF CHAR;
    a 		: REAL;

BEGIN 
  a:= parse2();
  WHILE (exprs[now]#NULL) DO
    gettoken(tok);
    IF ((tok[0]="+") OR (tok[0]="-")) THEN
      a:= binary(a,parse2(),tok);
    ELSE						(* IF 1		*)
      DEC(now);
      RETURN a;
    END;						(* IF 1		*)
  END;							(* WHILE 1	*)
  RETURN a;
END parse0;


BEGIN
  bool:= TRUE;
  Xres:=X;
  arglen:= Length(argument);
  FOR lauf:= 1 TO arglen DO
    exprs[lauf]:= CAP(argument[lauf-1]);
  END;
  exprs[0]:= NULL;
  exprs[arglen+1]:=NULL;
  now:=1;
  ans:= parse0();
  IF exprs[1]=NULL THEN
    bool:= FALSE;
    RETURN 1.00;
  END; 
  RETURN ans;
END LGSparse;

END LGSparser.
