(*---------------------------------------------------------------------------
   :Program.    Kurve
   :Author.     Jrg Wesemann
   :Address.    Auf der Heide 10, 2807 Achim - Baden
   :Phone.      (0)4202 / 70658
   :Shortcut.   [jwe]
   :Version.    2.94
   :Date.       23.03.89
   :Copyright.  PD
   :Language.   Modula-II
   :Translator. M2Amiga
   :Imports.    JWELib: Parser , Fastnull-Funktion
   :Update.     none.
   :Contents.   Definiert globale Variablen, Konstanten und Funktionen
   :Remark.     none.
---------------------------------------------------------------------------*)


IMPLEMENTATION MODULE startupproc;

FROM SYSTEM 		IMPORT 	FFP,ADR;
FROM FFPInOut 		IMPORT	WriteReal;
FROM InOut 		IMPORT 	Write,WriteString,WriteLn,WriteCard,WriteInt,
				OpenOutput,CloseOutput;
FROM JWELib 		IMPORT 	parse,fastnull;
FROM Intuition 		IMPORT 	CloseScreen,CloseWindow;  
FROM Strings 		IMPORT 	Length;  
FROM menu 		IMPORT 	wartemenu;

  CONST
    h1 = 0.005;
    h2 = 0.04;
    h3 = 0.08;


PROCEDURE cleanup;

BEGIN
  IF wp#NIL THEN
    CloseWindow(wp);
  END;
  IF sp#NIL THEN
    CloseScreen(sp);
  END;
END cleanup;



PROCEDURE crlf(anzahl: CARDINAL);

  VAR
    lauf: CARDINAL;

BEGIN
  FOR lauf:= 1 TO anzahl DO
    WriteLn;
  END;
END crlf;



PROCEDURE cursorr(anzahl: CARDINAL);

  VAR
    lauf: CARDINAL;

BEGIN
  FOR lauf:= 1 TO TRUNC(FLOAT(anzahl)/s) DO
    WriteString(spaces);
  END;
END cursorr;



PROCEDURE center(zeile: ARRAY OF CHAR);

BEGIN
  cursorr(38-CARDINAL(Length(zeile)/2));
  WriteString(zeile);
END center;



PROCEDURE f(x: FFP):FFP;

BEGIN
  RETURN parse(string,x);
END f;


PROCEDURE fn(x: FFP; k: CARDINAL):FFP;

  VAR
    fn : FFP;
    
BEGIN
  CASE k OF
    | 0: fn:= f(x); 
    | 1: fn:= (f(x+h1)-f(x-h1))/(2.0*h1);
    | 2: fn:= (f(x+h2)+f(x-h2)-2.0*f(x))/(h2*h2);
    | 3: fn:= ( f(x+h2+h3)-2.0*f(x+h3)+f(x-h2+h3)-f(x-h3+h2)+2.0*f(x-h3)
                -f(x-h2-h3) )/(2.0*h3*h2*h2);
  END;
  RETURN fn;
END fn;



PROCEDURE solve(xmin,xmax,dx: FFP; k,art: CARDINAL; VAR success: BOOLEAN):FFP;

  VAR
    xalt,x,xneu,y,yalt,yneu,nulleps,z : FFP;  

PROCEDURE nullstelle;
  
BEGIN    
  LOOP
    x:= (xneu-xmin)/2.0+xmin;
    y:= fn(x,k);
    IF ABS(y)<nulleps THEN
      EXIT;
    END;
    IF (y*yneu) >= eps THEN
      yneu:= y;
      xneu:= x;
    ELSE
      yalt:= y;
      xmin:= x;
    END;
  END;
END nullstelle;



BEGIN
  success:= FALSE;
  CASE k OF
    |  0 : nulleps:= 0.00001;
    |  1 : nulleps:= 0.00001;
    |  2 : nulleps:= 0.00002;
  ELSE
    nulleps:= eps;
  END;
  IF art=0 THEN
    x:= xmin;
    WHILE (x<xmax) AND (NOT success) DO
      y:= fn(x,k);
      REPEAT
        yalt:= y;
        xmin:= x;
        x   := xmin+dx;
        y:= fn(x,k);
      UNTIL ( (y * yalt) < -eps ) OR (x>= xmax);
      IF x<= xmax THEN
        yneu:= y;
        xneu:= x;
        nullstelle;
        success:= TRUE;
      END;				(* IF 		*)
    END;				(* WHILE 	*)
    RETURN x;
  ELSE					(* art # 0	*)
    x:= xmin + dx;
    yalt:= fn(xmin,k);
    xalt:= xmin;
    IF (yalt < -nulleps) OR (yalt > nulleps) THEN
      REPEAT
        y:= fn(x,k);
        IF y*yalt <= nulleps THEN
          success:= TRUE;
          IF (y < -nulleps) OR (y > nulleps) THEN
            REPEAT
              z:= x;
              IF fastnull(y-yalt) THEN
                x:= x + 0.0001;
              ELSE
                x:= x-y*(x-xalt)/(y-yalt);
              END;				(* IF 5		*)
              xalt:= z;
              yalt:= y;
              y:= fn(x,k);
            UNTIL ABS(x-xalt) < eps;		(* REPEAT 2	*)
          END;					(* IF 4		*)
        ELSE					(* y*yalt > 0.0 *)
          yalt:= y;
          xalt:= x;
          x:= x+dx;
        END;					(* IF 3		*)
      UNTIL ((x>xmax) OR success=TRUE);		(* REPEAT 1	*)
    ELSE					(* yalt = 0	*)
      success:= TRUE;
      x:= x-dx;
    END;					(* IF 2		*)
    RETURN x;
  END;						(* IF 1		*)
END solve;


PROCEDURE konstlist;

  VAR
    dummy : CARDINAL;

BEGIN
  Write(CLS); 
  IF dateiabfrage THEN
    center("Bitte geben Sie die Ausgabedatei sorgfltig ein !");WriteLn;WriteLn;
    cursorr(38); OpenOutput("kurve");
    dateiabfrage:= FALSE;
  END;   
  crlf(5); cursorr(20);
  WriteString("Auflistung der Programmkonstanten"); WriteLn; cursorr(20);
  WriteString("---------------------------------"); crlf(4); cursorr(20);
  WriteString("eps       : "); WriteReal(eps,10,8); WriteLn; cursorr(20);
  WriteString("h1        : "); WriteReal(h1,10,8); WriteLn; cursorr(20);
  WriteString("h2        : "); WriteReal(h2,10,8); WriteLn; cursorr(20);
  WriteString("h3        : "); WriteReal(h3,10,8); WriteLn; cursorr(20);
  WriteString("d         : "); WriteReal(d,10,8); WriteLn; cursorr(20);
  WriteString("m         : "); WriteInt(m,10); WriteLn; cursorr(20);
  WriteString("n         : "); WriteInt(n,10); WriteLn; cursorr(20);
  WriteString("MaxIt     : "); WriteCard(maxit,10); WriteLn; cursorr(20);
  WriteString("NoPoints  : "); WriteCard(nopoints,10); WriteLn; cursorr(20);
  crlf(2); 
  CloseOutput;
  center("Bitte 'RETURN' drcken fr MENU ");
  wartemenu;
END konstlist;


PROCEDURE simpson(a,b: FFP; k: CARDINAL):FFP;

  VAR
    h,s  	: FFP;
    lauf	: CARDINAL;

BEGIN
  h:= (b-a)/(2.0*FFP(k));
  s:= fn(a,0) + fn(b,0);
  FOR lauf:= 0 TO k-1 DO
    s:= s + 4.0 * fn(a+(2.0*FFP(lauf)+1.0)*h,0);
  END;
  FOR lauf:= 1 TO k-1 DO
    s:= s + 2.0 * fn(a+2.0*FFP(lauf)*h,0);
  END;
  s:= s * h / 3.0;
  RETURN s;
END simpson;


PROCEDURE intf(a,b,err: FFP;VAR maxbool: BOOLEAN):FFP;

  VAR
    i,lasti,realerr	: FFP;
    k			: CARDINAL;
    
BEGIN
  k:= 4;
  lasti:= simpson(a,b,2);
  REPEAT
    i:= simpson(a,b,k);
    IF fastnull(i) THEN
      realerr:= err;
    ELSE
      realerr:= ABS((i-lasti)/i);
    END;
    lasti:= i;
    INC(k,k);
  UNTIL (realerr<=err) OR (k>maxit);
  IF k>maxit THEN
    maxbool:= TRUE;
  ELSE
    maxbool:= FALSE;
  END;
  RETURN(i); 
END intf;


PROCEDURE fragedatei();

BEGIN
  center("Bitte geben Sie die Ausgabedatei sorgfltig ein !");
  crlf(3);
  cursorr(34);
  OpenOutput("Kurve");
END fragedatei;


END startupproc.



