(*---------------------------------------------------------------------------
    :Program.    FileSystem.mod
    :Author.     ??? backtranslated: Bernd Preusing
    :Address.    Gerhardstr. 16  D-2200 Elmshorn
    :Phone.      04121/22486
    :Shortcut.   [bep]
    :Version.    
    :Date.       
    :Copyright.  PD
    :Language.   Modula-II
    :Translator. M2Amiga
    :Imports.    
    :UpDate.     
    :Contents.   
    :Remark.     FEHLER sind gekennzeichnet (* 'ne ganze Menge! *)
---------------------------------------------------------------------------*)
IMPLEMENTATION MODULE FileSystem;

FROM SYSTEM IMPORT ADR, ADDRESS, BYTE, CAST;

IMPORT Dos;

FROM Exec IMPORT
	MemReqs, MemReqSet,  FreeMem, AllocMem, CopyMem;

FROM Arts IMPORT
	CurrentLevel, TermProcedure;

FROM ASCII IMPORT
	nul;


TYPE
  (* CHARPtr = POINTER TO CHAR; *)
  ExtFilePtr = POINTER TO ExtFile;
  ExtFile = RECORD
		next: ExtFilePtr;
		size: LONGINT;
		level: INTEGER;
		fname: ARRAY[0..63] OF CHAR;
		mfile: File;
		(* buffer: ARRAY[0..MAX(CARDINAL)] OF CHAR; *)
	    END;

VAR
  FileList: ExtFilePtr;


PROCEDURE AddFile(f: ExtFilePtr);
BEGIN
  f^.next:=FileList;
  f^.level:=CurrentLevel();
  FileList:=f
END AddFile;

PROCEDURE KillFile(VAR f: File);
VAR a,b: ExtFilePtr;
BEGIN
  WITH f DO
    IF bufa#NIL THEN
      a:=CAST(ExtFilePtr, CAST(LONGINT,bufa)-SIZE(ExtFile));
      IF a=FileList THEN
        FileList:=FileList^.next;
        FreeMem(a,a^.size)
      ELSE
        IF FileList#NIL THEN
          b:=FileList;
          WHILE (b#NIL) AND (b^.next#a) DO
            b:=b^.next
          END;
          IF b#NIL THEN
            b^.next:=a^.next;
            FreeMem(a,a^.size)
          END
        END
      END;
      bufa:=NIL;
      ela:=NIL;
      ina:=NIL;
      topa:=NIL;
      filePos:=0;
      bufPos:=0;
    END;
  END
END KillFile;

PROCEDURE UpdateMemFile(VAR f: File);
VAR a: ExtFilePtr;
BEGIN
  a:=CAST(ExtFilePtr,CAST(LONGINT,f.bufa)-SIZE(ExtFile));
  a^.mfile:=f
END UpdateMemFile;

(* wre viel krzer bei SetRes *)
  CASE err OF
    Dos.noFreeStore:		res:=memErr |
    Dos.objectInUse:		res:=inUse  |
    Dos.objectNotFound:		res:=notFound |
    Dos.diskWriteProtected:	res:=diskWriteProtected |
    Dos.deviceNotMounted:	res:=deviceNotMounted |
    Dos.seekError:		res:=seekErr |
    Dos.diskFull:		res:=diskFull |
    Dos.deleteProtected:	res:=deleteProtected |
    Dos.writeProtected:		res:=writeProtected |
    Dos.notADosDisk:		res:=notDosDisk |
    Dos.noDisk:			res:=noDisk |
*)
PROCEDURE SetRes(VAR res:Response; default:Response);
VAR err:INTEGER;
BEGIN
  err:=Dos.IoErr();
  IF    err=Dos.noFreeStore THEN res:=memErr
  ELSIF err=Dos.objectInUse THEN res:=inUse
  ELSIF err=Dos.objectNotFound THEN res:=notFound
  ELSIF err=Dos.diskWriteProtected THEN res:=diskWriteProtected
  ELSIF err=Dos.deviceNotMounted THEN res:=deviceNotMounted
  ELSIF err=Dos.seekError THEN res:=seekErr
  ELSIF err=Dos.diskFull THEN res:=diskFull
  ELSIF err=Dos.deleteProtected THEN res:=deleteProtected
  ELSIF err=Dos.writeProtected THEN res:=writeProtected
  ELSIF err=Dos.notADosDisk THEN res:=notDosDisk
  ELSIF err=Dos.noDisk THEN res:=noDisk
  ELSE  res:=default
  END
END SetRes;

PROCEDURE Readf(VAR f: File; adr: ADDRESS; len: LONGINT; VAR actual:LONGINT);
BEGIN
  WITH f DO
    actual:=Dos.Read(file,adr,len);
    IF actual=-1 THEN
      SetRes(res,readErr)
    ELSE
      eof:=(actual#len)
    END
  END
END Readf;

PROCEDURE Writef(VAR f: File; adr: ADDRESS; len: LONGINT; VAR actual:LONGINT);
BEGIN
  WITH f DO
    actual:=Dos.Write(file,adr,len);
    IF actual=-1 THEN
      SetRes(res,writeErr)
    END
  END
END Writef;

PROCEDURE Flush(VAR f:File);
VAR a,b:LONGINT;
BEGIN
  WITH f DO
    IF write IN mode THEN
      IF filePos#bufPos THEN
        a:=Dos.Seek(file,bufPos,Dos.beginning)
      ELSE
	a:=0
      END;
      IF a#-1 THEN
        b:=CAST(LONGINT,ina)-CAST(LONGINT,bufa);
	a:=Dos.Write(file,bufa,b);
	IF a#b THEN
	  res:=writeErr
	ELSE
	  INC(bufPos,b);
	  filePos:=bufPos
	END
      ELSE
        res:=seekErr
      END;
      mode:=mode-FileModeSet{write}
    END;
    ela:=bufa;
    ina:=bufa;
    IF (res=done) AND (read IN mode) THEN
      b:=CAST(LONGINT,topa)-CAST(LONGINT,bufa);
      a:=Dos.Read(file,bufa,b);
      IF a>=0 THEN
        bufPos:=filePos;
	INC(filePos,a);
	ina:=CAST(CHARPtr,CAST(LONGINT,bufa)+a)
      ELSE
        res:=readErr
      END
    END
  END (* with *)
END Flush;

PROCEDURE Lookup(VAR f: File; name: ARRAY OF CHAR;
		 bufferSize: CARDINAL; new: BOOLEAN);
VAR a, actmode:LONGINT;  i: INTEGER; ep: ExtFilePtr;
BEGIN
  a:=LONGINT(bufferSize)+SIZE(ExtFile);
  ep:=AllocMem(a,MemReqSet{memClear});
  IF ep#NIL THEN
    ep^.size:=a;
    WITH ep^ DO
      i:=0;
      WHILE (i<63) AND (i<=HIGH(name)) AND (name[i]#0C) DO
        fname[i]:=name[i];
	INC(i)
      END;
      fname[i]:=0C;
      WITH mfile DO
        res:=done;
	IF new THEN
	  actmode:=Dos.newFile
	ELSE
	  actmode:=Dos.oldFile
	END;
	file:=Dos.Open(ADR(fname),actmode);
	IF file=NIL THEN
	  SetRes(res,openErr)
	END;
	bufa:=CAST(CHARPtr,CAST(LONGINT,ep)+SIZE(ExtFile));
	topa:=CAST(CHARPtr,CAST(LONGCARD,bufa)+LONGCARD(bufferSize));
	IF bufferSize#0 THEN
	  mode:=FileModeSet{}
	ELSE
	  mode:=FileModeSet{noBuffer}
	END;
	ina:=bufa;
	ela:=bufa;
	filePos:=0;
	bufPos:=0;
	f:=mfile;
	IF res=done THEN
	  AddFile(ep)
	ELSE
	  FreeMem(ep,ep^.size)
	END
      END; (* with mfile (viel zu spt!) *)
    END (* with ep^ *)
  ELSE
    f.res:=memErr
  END
END Lookup;

PROCEDURE Close(VAR f: File);
BEGIN
  WITH f DO
    IF file#NIL THEN
      res:=done;
      IF write IN mode THEN
        mode:=FileModeSet{write};
        Flush(f)
      END;
      IF res=done THEN
        Dos.Close(file);
        file:=NIL;
        KillFile(f)
      END
    ELSE
      KillFile(f)
    END
  END
END Close;

PROCEDURE Delete(VAR f: File);
VAR a:ExtFilePtr; ok:BOOLEAN;
BEGIN
  WITH f DO
    a:=CAST(ExtFilePtr,CAST(LONGINT,bufa)-SIZE(ExtFile));
    Dos.Close(file);
    file:=NIL;
    ok:=(Dos.DeleteFile(ADR(a^.fname)) # 0);
    KillFile(f);
    IF ok THEN
      res:=done
    ELSE
      SetRes(res,notdone)
    END
  END
END Delete;

PROCEDURE SetPos(VAR f: File; pos: LONGINT);
VAR seek:LONGINT;
BEGIN
  WITH f DO
    res:=done;
    IF noBuffer IN mode THEN
      seek:=Dos.Seek(file,pos,Dos.beginning);
      IF seek=-1 THEN
        SetRes(res,seekErr)
      END
    ELSE
      IF (bufPos<=pos) AND
		(pos < (CAST(LONGINT,ina)-CAST(LONGINT,bufa)+bufPos)) THEN
        ela:=CAST(CHARPtr,CAST(LONGINT,bufa)+pos-bufPos)
      ELSE
        mode:=mode-FileModeSet{read};
        Flush(f);
        seek:=Dos.Seek(file,pos,Dos.beginning);
        IF seek#-1 THEN
          filePos:=pos;
          bufPos:=pos;
          ela:=bufa;
          ina:=bufa
        ELSE
          SetRes(res,seekErr)
        END
      END
    END;
  END;
  UpdateMemFile(f);
END SetPos;

PROCEDURE GetPos(VAR f: File; VAR pos: LONGINT);
BEGIN
  WITH f DO
    IF noBuffer IN mode THEN
      pos:=Dos.Seek(file,0,Dos.current);
      IF pos=-1 THEN
        SetRes(res,seekErr)
      END
    ELSE
      pos:=CAST(LONGINT,ela)-CAST(LONGINT,bufa)+bufPos;
      res:=done
    END
  END
END GetPos;

PROCEDURE Length(VAR f: File; VAR pos: LONGINT);
VAR OldPos: LONGINT;
BEGIN
  WITH f DO
    OldPos:=Dos.Seek(file,0,Dos.end);
    pos:=Dos.Seek(file,OldPos,Dos.beginning);
    IF write IN mode THEN (* falls noch etwas im Puffer *)
      OldPos:=bufPos+CAST(LONGINT,ina)-CAST(LONGINT,bufa);
      IF OldPos>pos THEN
        pos:=OldPos
      END
    END
  END
END Length;

PROCEDURE ReadChar(VAR f: File; VAR ch: CHAR);
VAR actual: LONGINT;
BEGIN
  WITH f DO
    IF res=done THEN
      IF noBuffer IN mode THEN
        Readf(f,ADR(ch),1,actual)
      ELSE
        mode:=mode+FileModeSet{read};
        IF ela=ina THEN Flush(f) END;
        eof:=(ela=ina);
        IF (res=done) AND (NOT eof) THEN
          ch:=ela^;
          INC(CAST(LONGINT,ela)) (* verkrzt den Code: TRAPV statt TRAP #15 *)
        ELSE
          ch:=nul
        END;
        UpdateMemFile(f)
      END
    END
  END
END ReadChar;

PROCEDURE WriteChar(VAR f: File; ch: CHAR);
VAR actual: LONGINT;
BEGIN
  WITH f DO
    IF res=done THEN
      IF noBuffer IN mode THEN
        Writef(f,ADR(ch),1,actual)
      ELSE
        mode:=mode+FileModeSet{write};
        IF ela=topa THEN
          Flush(f);
        END;
        IF res=done THEN
          ela^:=ch;
          INC(CAST(LONGINT,ela));
          IF CAST(LONGINT,ela)>CAST(LONGINT,ina) THEN
            ina:=ela
          END
        END;
        UpdateMemFile(f)
      END
    END
  END
END WriteChar;

PROCEDURE ReadBytes(VAR f: File; adr: ADDRESS;
			len: LONGINT; VAR actual: LONGINT);
VAR toCopy: LONGINT;
BEGIN
  WITH f DO
    IF noBuffer IN mode THEN
      Readf(f,adr,len,actual)
    ELSE
      actual:=0;
      LOOP
        mode:=mode+FileModeSet{read};
	toCopy:=CAST(LONGINT,ina)-CAST(LONGINT,ela);
	IF toCopy>len THEN toCopy:=len END;
	CopyMem(ela,adr,toCopy);
	DEC(len,toCopy);
	INC(actual,toCopy);
	INC(CAST(LONGINT,ela),toCopy);
	INC(adr,toCopy);
	IF len=0 THEN
	  EXIT
	ELSE
	  Flush(f);
	  IF eof OR (res#done) THEN EXIT END;
	END;
      END; (* loop *)
    END; (* if noBuffer *)
    UpdateMemFile(f);
  END; (* with f *)
END ReadBytes;

PROCEDURE WriteBytes(VAR f: File; adr: ADDRESS;
			len: LONGINT; VAR actual: LONGINT);
VAR toCopy: LONGINT;
BEGIN
  WITH f DO
    IF noBuffer IN mode THEN
      Writef(f,adr,len,actual)
    ELSE
      actual:=0;
      LOOP
        mode:=mode+FileModeSet{write};
	toCopy:=CAST(LONGINT,topa)-CAST(LONGINT,ela);
	IF toCopy>len THEN toCopy:=len END;
	CopyMem(adr,ela,toCopy);
	DEC(len,toCopy);
	INC(actual,toCopy);
	INC(CAST(LONGINT,ela),toCopy);
	INC(adr,toCopy);
	IF CAST(LONGINT,ela)>CAST(LONGINT,ina) THEN ina:=ela END;
	IF len=0 THEN
	  EXIT
	ELSE
	  Flush(f);
	  IF res#done THEN EXIT END;
	END;
      END; (* loop *)
    END; (* if noBuffer *)
  END; (* with f *)
  (* FEHLER: UpdateMemFile(f) fehlt hier !!! *)
END WriteBytes;

PROCEDURE ReadByteBlock(VAR f: File; VAR block: ARRAY OF BYTE);
VAR actual: LONGINT;
BEGIN
  ReadBytes(f,ADR(block),HIGH(block)+1,actual)
  (* war FEHLER: hier stand im neuesten Update: HIGH(block)-1 !!! *)
END ReadByteBlock;

PROCEDURE WriteByteBlock(VAR f: File; VAR block: ARRAY OF BYTE);
VAR actual: LONGINT;
BEGIN
  WriteBytes(f,ADR(block),HIGH(block)+1,actual)
  (* war FEHLER: hier stand im neuesten Update: HIGH(block)-1 !!! *)
END WriteByteBlock;

PROCEDURE CloseFiles();
BEGIN
  WHILE (FileList#NIL) AND (FileList^.level<=CurrentLevel()) DO
  (* FEHLER: mu lauten: >=CurrentLevel() !!! *)
    Close(FileList^.mfile)
  END;
END CloseFiles;

BEGIN
  FileList:=NIL;
  TermProcedure(CloseFiles)
END FileSystem.mod
