MODULE FileUpdt;   (* Update two files using random access *)

  (* From the book "Modula-2  A Seafarer's Manual and Shipyard Guide" *)
  (* Page 142   adapted "Amiga M2Modula-2"   09 Mar 1988 *)

(* terminal input/ouput *)
FROM InOut IMPORT ReadCard,		(* read cardinal value *)
                  ReadString,		(* read string *)
                  WriteLn,		(* write cr/line feed *)
                  WriteString;		(* write string *)
                  
(* file input & output tools *)
FROM FileSystem IMPORT File,		(* file definition *)
                       Response,	(* includes 'done' *)
                       Lookup,		(* look up file on disk *)
                       SetPos,		(* set position *)
                       Close,		(* close *)
                       ReadChar,	(* read character *)
                       WriteChar;	(* write character *)
                       
CONST
  LenCrData = 35;	(* length of data per crew member in file,
                           includes carriage return/line feed *)
  LenCrNum = 8;		(* length of crew mem number in file *)
  
VAR
  FL1,
  FL2 : File;				(* contains file definition &
                                           status info *)
  CrewNum : [1..100];			(* crew mem. number *)
  JobDes : ARRAY [0..25] OF CHAR;	(* job description *)
  inputnum : CARDINAL;			(* variable to input cardinal value *)
    
(* Initialize file *)
PROCEDURE InitFL (VAR FL : File;		(* file defin. *)
                  FLName : ARRAY OF CHAR);	(* file name *)
                  
  BEGIN
    Lookup (FL,				(* look up file on disk *)
            FLName,
            2048,			(* buffersize *)
            FALSE);
  END InitFL;


(* Read description from file for specified crew member *)
PROCEDURE ReadDes (VAR FL : File;		(* file defin. *)
                   CrNum : CARDINAL;		(* crew mem num *)
                   VAR Des : ARRAY OF CHAR);	(* job des. *)

  VAR
    i : CARDINAL;			(* array index *)
    ch : CHAR;				(* holds character *)
    
  BEGIN					(* set file position *)
    SetPos (FL,				(* file *)
            (((CrNum-1)*LenCrData)+LenCrNum));	(* character position *)
    IF (FL.res = done) THEN		(* skip if error *)
      i := 0;
      REPEAT				(* read description *)
        ReadChar (FL,			(* char by char *)
                  ch);
        Des[i] := ch;			(* abort if error *)
        INC (i);
      UNTIL ((i > CARDINAL (HIGH (Des))) OR (FL.res # done));
    END;   (* IF *)
  END ReadDes;
  

(* Update description on file for specified crew member *)
PROCEDURE UpdtFL (VAR FL : File;		(* file defin. *)
                  CrNum : CARDINAL;		(* crew mem num *)
                  Des : ARRAY OF CHAR);		(* job des. *)

  VAR
    i : CARDINAL;			(* array index *)
    ch : CHAR;				(* holds character *)
    
  BEGIN
    SetPos (FL,				(* file *)
            (((CrNum-1)*LenCrData)+LenCrNum));	(* character position *)
    IF (FL.res = done) THEN		(* skip if error *)
      i := 0;
      REPEAT				(* write description *)
        ch := Des[i];			(* char by char *)
        WriteChar (FL,
                   ch);
        INC (i);
      UNTIL ((i > CARDINAL (HIGH (Des))) OR (FL.res # done));
    END;   (* IF *)
  END UpdtFL;
  

BEGIN   (* main body *)
  WriteLn;
  WriteString ("Start file update");
  
  InitFL (FL1,					(* initialize first file *)
          "CREW1.TXT");				(* drive/path/file name *)
  InitFL (FL2,					(* initialize second file *)
          "CREW2.TXT");				(* drive/path/file name *)
  IF ((FL1.res = done) AND (FL2.res = done)) THEN   (* error? *)
    LOOP					    (* no *)
      WriteLn;
      WriteString ("Enter crew number: ");
      ReadCard (inputnum);
      IF (inputnum = 0) THEN		(* crew num = 0? *)
        EXIT;				(* yes - quit *)
      END;   (* IF *)
      CrewNum := inputnum;
      
      ReadDes (FL1,CrewNum,JobDes);	(* read des. from file *)
      IF (FL1.res # done) THEN		(* error? *)
        EXIT;				(* yes *)
      END;   (* IF *)
      
      WriteLn;
      WriteString ("old job description: ");
      WriteString (JobDes);
      
      WriteLn;
      WriteString ("enter new job description: ");
      ReadString (JobDes);
      
      UpdtFL (FL1,CrewNum,JobDes);	(* update des. on file 1 *)
      IF (FL1.res # done) THEN		(* error? *)
        EXIT;				(* yes *)
      END;   (* IF *)
      
      UpdtFL (FL2,CrewNum,JobDes);	(* update des. on file 2 *)
      IF (FL2.res # done) THEN		(* error? *)
        EXIT;
      END;   (* IF *)
      
    END;   (* LOOP *)
    
  END;   (* IF done *)
  
  IF (FL1.res = done) THEN		(* error on file? *)
    Close (FL1);			(* no - close it *)
  END;
  
  IF (FL2.res = done) THEN		(* error on file? *)
    Close (FL2);			(* no - close it *)
  END;
  
  IF ((FL1.res # done) OR (FL2.res # done)) THEN
    WriteLn;				(* error encountered *)
    WriteString ("error encountered while accessing file");
  END;
  
  WriteLn;
  WriteString ("End file update");
  WriteLn;
  
END FileUpdt.
