MODULE SortAgePntr;   (* Build sorted age list using pointers *)

  (* From the book "Modula-2  A Seafarer's Manual and Shipyard Guide" *)
  (* Page 149   adapted "Amiga M2Modula-2"   09 Mar 1988 *)

FROM InOut IMPORT WriteLn,
                  WriteString,
                  WriteCard,
                  ReadCard;
FROM Storage IMPORT ALLOCATE;
                                   
TYPE
  AgePntr = POINTER TO AgeNode;
  AgeNode =			(* list entry, age/next pointer *)
    RECORD
      CrewMemAge : CARDINAL;
      NextAge : AgePntr;
    END;
    
VAR
  YoungPntr,			(* list pointers: youngest age *)
  NewPntr,			(*                new age entry *)
  CurrPntr,			(*                current one for search *)
  PrevPntr : AgePntr;		(*                previous one for search *)
  
BEGIN
  YoungPntr := NIL;		(* initially set list null *)
  WriteLn;
  WriteString ("Enter crew ages: ");
  WriteLn;
  
  LOOP
    ALLOCATE (NewPntr, SIZE (NewPntr^));   (* allocate space for list entry *)
    WriteLn;
    ReadCard (NewPntr^.CrewMemAge);	(* accept age from keyboard *)
    IF (NewPntr^.CrewMemAge = 0) THEN	(* age = 0? *)
      EXIT;				(* yes - input done *)
    END;   (* IF *)			(* no - add age in list by
                                           ascending order *)
    PrevPntr := NIL;		(* set previous pointer to NIL *)
    CurrPntr := YoungPntr;	(* set current pointer to youngest *)
    
    (* find place in list where age belongs *)
    WHILE ((CurrPntr # NIL) AND
           (NewPntr^.CrewMemAge > CurrPntr^.CrewMemAge))
           DO
      PrevPntr := CurrPntr;
      CurrPntr := CurrPntr^.NextAge;
    END;   (* WHILE *)		(* quit searching if end of list or new
                                   age <= current age on list *)
                                   
    IF (CurrPntr # NIL) THEN	(* new age <= current age on list? *)
      NewPntr^.NextAge := CurrPntr;	(* yes - put new in front *)
    ELSE
      NewPntr^.NextAge := NIL;		(* no - put new at end *)
    END;   (* IF *)
    
    IF (PrevPntr # NIL) THEN	(* is new before youngest or is list null? *)
      PrevPntr^.NextAge := NewPntr;	(* no - point previous at new *)
    ELSE
      YoungPntr := NewPntr;		(* yes - point youngest at new *)
    END;   (* IF *)
    
  END;   (* LOOP *)		(* get next age *)
  
  WriteLn;
  WriteString ("Sorted ages:");
  WriteLn;
  
  CurrPntr := YoungPntr;		(* display ages in order *)
  WHILE (CurrPntr # NIL) DO		(* end of list? *)
    WriteLn;				(* no *)
    WriteCard (CurrPntr^.CrewMemAge,0);
    CurrPntr := CurrPntr^.NextAge;	(* point to next one *)
  END;   (* WHILE *)
  
END SortAgePntr.
