(**********************************************************************

    :Program.    List.mod
    :Contents.   Generic data type: List
    :Author.     Michael Frie
    :Address.    Kernerstr. 22a
    :Copyright.  Public Domain, (C) Copyright 1988 by Michael Frie
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    TaskMemory [bne]
    :History.    V1.0 [mif] 13.Sep.1988
    :History.    V1.1 [bne] 8.Feb.1989 (+AllocProc,ListOk(),Init():BOOLEAN)

**********************************************************************)

IMPLEMENTATION MODULE List;
(* MODULE List: (C) Copyright 1988 by Michael Frie *)

FROM SYSTEM    IMPORT BYTE, ADDRESS, ADR;
FROM TaskMemory IMPORT Allocate, Deallocate;
FROM Exec      IMPORT CopyMem;

CONST Copyright = "MODULE List: (C) Copyright 1988 by Michael Frie";

TYPE list       = POINTER TO listbody;
     nodeptr = POINTER TO node;
     node    = RECORD
                   Data : ADDRESS;
                   Next : nodeptr;
                  END;
       listbody = RECORD
                   Top, Act : nodeptr;
                   Ok:BOOLEAN;
                  END;

PROCEDURE Init (VAR l:list):BOOLEAN;
 BEGIN
  ListAllocProc (l, SIZE(listbody));
  IF l # NIL THEN
    ListAllocProc (l^.Top, SIZE(node));
    IF l^.Top # NIL THEN
      l^.Act := l^.Top;
      l^.Act^.Next := NIL;
      l^.Ok:=TRUE;
      RETURN TRUE;
    END;
    ListDeallocProc(l);
  END;
  RETURN FALSE;
 END Init;


PROCEDURE Reset (l:list);
 BEGIN
  l^.Act := l^.Top
 END Reset;


PROCEDURE Insert (l:list; Data : ARRAY OF BYTE);

 VAR p : nodeptr;

 BEGIN
  ListAllocProc (p, SIZE(node));
  IF p # NIL THEN
    ListAllocProc(p^.Data, HIGH(Data)+1);
    IF p^.Data # NIL THEN
      IF l^.Act^.Next = NIL THEN
        p^.Next := NIL;
        l^.Act^.Next := p;
        p := l^.Act
      ELSE
        p^.Next := l^.Act^.Next;
        l^.Act^.Next := p;
        l^.Act := p
      END;
      CopyMem (ADR(Data), p^.Data, HIGH(Data)+1);
      RETURN; (* Ok=TRUE *)
    ELSE
      ListDeallocProc(p);
    END;
  END;
  l^.Ok:=FALSE;
 END Insert;


PROCEDURE Write  (l:list; Data : ARRAY OF BYTE);
 BEGIN
  IF l^.Act^.Next = NIL THEN
   ListAllocProc (l^.Act^.Next, SIZE(node));
   IF l^.Act^.Next # NIL THEN
     l^.Act^.Next^.Next := NIL;
     ListAllocProc (l^.Act^.Data, HIGH(Data)+1);
     IF l^.Act^.Data = NIL THEN
       l^.Ok:=FALSE;
       ListDeallocProc(l^.Act^.Next);
       RETURN;
     END;
   ELSE
     l^.Ok:=FALSE;
     RETURN
   END;
  END;
  CopyMem (ADR(Data), l^.Act^.Data, HIGH(Data)+1);
  l^.Act := l^.Act^.Next
 END Write;


PROCEDURE Read (l:list; VAR Data : ARRAY OF BYTE);
 BEGIN
  IF l^.Act^.Next # NIL THEN
   CopyMem (l^.Act^.Data, ADR(Data), HIGH(Data)+1);
   l^.Act := l^.Act^.Next;
  END
 END Read;

PROCEDURE Delete (l:list);
 VAR p : nodeptr;
 BEGIN
  IF (l^.Act^.Next # NIL) THEN
   ListDeallocProc (l^.Act^.Data);
   l^.Act^.Data := l^.Act^.Next^.Data;
   p := l^.Act^.Next^.Next;
   ListDeallocProc (l^.Act^.Next);
   l^.Act^.Next := p
  END
 END Delete;

PROCEDURE End (l:list) : BOOLEAN;
 BEGIN
  RETURN l^.Act^.Next = NIL
 END End;

PROCEDURE Do (l:list; Action : action);
 VAR More : BOOLEAN;
 BEGIN
  More := TRUE;
  WHILE (l^.Act^.Next # NIL) AND More DO
   Action (l^.Act^.Data^, More);
   l^.Act := l^.Act^.Next
  END
 END Do;

PROCEDURE Discard (VAR l:list);

 BEGIN
  WHILE l^.Top^.Next # NIL DO
   l^.Act := l^.Top^.Next;
   ListDeallocProc (l^.Top^.Data);
   ListDeallocProc (l^.Top);
   l^.Top := l^.Act;
  END;
  ListDeallocProc (l^.Top);
  ListDeallocProc (l);
 END Discard;

PROCEDURE ListOk(l:list):BOOLEAN;
BEGIN
  RETURN l^.Ok;
END ListOk;

BEGIN
  ListAllocProc:=Allocate;
  ListDeallocProc:=Deallocate;
END List.
