(**********************************************************************

    :Program.    ObjScanner.mod
    :Contents.   Object code scanner for Make [bne]
    :Support.    thanks to Bernd Preusing for his description of
    :Support.    the obj-file format
    :Author.     Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Amok internal, NOT in the public domain !
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    ErrorReq1.2 [bne]
    :History.    V1.0a [bne] 6.Feb.1989
    :Update.     17.Feb.1989, 22.Feb.1989 [bne] cosmetics

**********************************************************************)

IMPLEMENTATION MODULE ObjScanner;

FROM Arts       IMPORT ModType,ModKeys,ModName;
FROM ErrorReq   IMPORT Assert;
FROM FileSystem IMPORT Lookup,Close,ReadChar,ReadBytes,ReadByteBlock,
                Response,File,GetPos,SetPos;
FROM SYSTEM     IMPORT ADR;

CONST   BufferSize=256;
        oldFile=FALSE;

        (* Error messages *)
        WrongFile       = "code file expected";
        NoModuleId      = "ModuleId expected";

VAR     ObjFile:File;
        ObjKey:ModKeys;
        NumImports:LONGINT; (* imported Modules + empty IDs *)

(**)
(* ModuleId: see M2Amiga Manual V3.2 page 10-6                          *)
(**)
PROCEDURE ReadModuleId(VAR Type:ModType;
                       VAR Name:ModName;
                       VAR Key:ModKeys):BOOLEAN;
VAR     TypeNum:INTEGER;
BEGIN
  ReadByteBlock(ObjFile,TypeNum);
  Type:=VAL(ModType,TypeNum);
  ReadByteBlock(ObjFile,Name);
  ReadByteBlock(ObjFile,Key);
  RETURN ObjFile.res=done;
END ReadModuleId;

PROCEDURE StartScan(FileName:ARRAY OF CHAR):BOOLEAN;
CONST   CODEFILE=LONGINT(1);
VAR     FileID,Pos:LONGINT;
        TestName:ModName; (* Dummy, always equal FileName (without path) *)
        ObjType:ModType;  (* Dummy, always equal mod (of course) *)
        CodeSize,DataSize,ConstSize,NumProcs:LONGINT;
BEGIN
  Lookup(ObjFile,FileName,BufferSize,oldFile);
  IF ObjFile.res=done THEN
    ReadByteBlock(ObjFile,FileID);
    Assert(FileID=CODEFILE,ADR(WrongFile));
    Assert(ReadModuleId(ObjType,TestName,ObjKey),ADR(NoModuleId));
    ReadByteBlock(ObjFile,CodeSize);
    ReadByteBlock(ObjFile,DataSize);
    ReadByteBlock(ObjFile,ConstSize);
    ReadByteBlock(ObjFile,NumProcs);
    ReadByteBlock(ObjFile,NumImports);
    GetPos(ObjFile,Pos);
    SetPos(ObjFile,Pos+ConstSize+6*NumProcs); (* skip ConstBlock *)
    IF ObjFile.res#done THEN
      NumImports:=0;
    END;
    RETURN TRUE;
  ELSE
    RETURN FALSE;
  END;
END StartScan;

PROCEDURE FindNextImport(VAR ModuleName:ModName;
                         VAR ModuleKey:ModKeys;
                         VAR ModuleType:ModType):BOOLEAN;
VAR     Ok:BOOLEAN;
BEGIN
  IF NumImports>0 THEN
    REPEAT
      Ok:=ReadModuleId(ModuleType,ModuleName,ModuleKey);
      DEC(NumImports);
    UNTIL (ModuleType#none)AND(ModuleType#lib) (* empty entries and libs are ignored *)
       OR (NumImports=0);    (* no more entries *)
    RETURN (ModuleType#none) AND Ok;
  ELSE
    RETURN FALSE;
  END;
END FindNextImport;

PROCEDURE EndScan(VAR ModuleKey:ModKeys);
BEGIN
  ModuleKey:=ObjKey;
  Close(ObjFile);
END EndScan;

END ObjScanner.

