(**********************************************************************

    :Program.    SymScanner.mod
    :Contents.   Symbol file scanner for Make [bne]
    :Author.     Nicolas Benezan [bne]
    :Address.    Postwiesenstr. 2, D7000 Stuttgart 60
    :Phone.      711/333679
    :Copyright.  Amok internal, NOT in the public domain !
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    ErrorReq1.2 [bne]
    :History.    V1.0a [bne] 5.Feb.1989
    :Update.     17.Feb.1989, 22.Feb.1989 [bne] cosmetics

**********************************************************************)

IMPLEMENTATION MODULE SymScanner;

FROM Arts       IMPORT ModType,ModKeys,ModName,maxModName;
FROM ErrorReq   IMPORT Assert;
FROM Exec       IMPORT Byte,UByte;
FROM FileSystem IMPORT File,Lookup,Close,Response,ReadByteBlock,ReadChar,
                ReadBytes,GetPos,SetPos;
FROM SYSTEM     IMPORT ADR;
IMPORT ASCII;

CONST   BufferSize=512;
        oldFile=FALSE;
        REFFILE=LONGINT(2);
        ModAnchor=Byte(0);

        (* Error messages *)
        CorruptAnchor   = "corrupt ModAnchor";
        WrongFile       = "symbol file expected";
        NoAnchor        = "missing ModAnchor";

VAR     SymKey:ModKeys;
        SymType:ModType;
        SymFile:File;

(**)
(* ModAnchor: see M2Amiga Manual V3.2 page 10-8                         *)
(**)
PROCEDURE FindModAnchor(VAR ModuleName:ARRAY OF CHAR;
                        VAR ModuleKey:ModKeys;
                        VAR ModuleType:ModType):BOOLEAN;
CONST   ModAnchorID=CHAR(0);
VAR     StructID:CHAR;
        Pos:LONGINT;
        Len,Count:UByte;
BEGIN
  ReadChar(SymFile,StructID);
  IF (SymFile.res=done) AND (StructID=ModAnchorID) THEN
    ReadByteBlock(SymFile,ModuleKey);
    ReadChar(SymFile,CHAR(Len));
    DEC(Len); (* the actual length is Len-1 *)
    Assert((Len>0)AND(Len<=maxModName),ADR(CorruptAnchor));
    ReadBytes(SymFile,ADR(ModuleName),Len,Pos);
    ModuleName[Len]:=ASCII.nul;
    ReadChar(SymFile,CHAR(ModuleType));
    IF ModuleType=lib THEN
      ReadChar(SymFile,CHAR(Len));
      GetPos(SymFile,Pos);
      SetPos(SymFile,Pos+LONGINT(Len)-1);
    END;
    RETURN SymFile.res=done;
  ELSE
    RETURN FALSE;
  END;
END FindModAnchor;

PROCEDURE StartScan(FileName:ARRAY OF CHAR):BOOLEAN;
VAR     FileID:LONGINT;
        TestName:ModName; (* Dummy *)
BEGIN
  Lookup(SymFile,FileName,BufferSize,oldFile);
  IF SymFile.res=done THEN
    ReadByteBlock(SymFile,FileID);
    Assert(FileID=REFFILE,ADR(WrongFile));
    Assert(FindModAnchor(TestName,SymKey,SymType),ADR(NoAnchor));
    (* the first ModAnchor is actually NOT an import but the module itself *)
    RETURN TRUE;
  ELSE
    RETURN FALSE;
  END;
END StartScan;

PROCEDURE FindNextImport(VAR ModuleName:ARRAY OF CHAR;
                        VAR ModuleKey:ModKeys;
                        VAR ModuleType:ModType):BOOLEAN;
VAR     Found:BOOLEAN;
BEGIN
  REPEAT
    Found:=FindModAnchor(ModuleName,ModuleKey,ModuleType)
  UNTIL (ModuleType#lib) OR NOT Found; (* libs are ignored *)
  RETURN Found;
END FindNextImport;

PROCEDURE EndScan(VAR ModuleType:ModType;VAR ModuleKey:ModKeys);
BEGIN
  ModuleType:=SymType;
  ModuleKey:=SymKey;
  Close(SymFile);
END EndScan;

END SymScanner.

