(**********************************************************************

    :Program.    Queue.mod
    :Contents.   Generic data type: Queue (FIFO-list)
    :Author.     Michael Frie [mif]
    :Address.    Kernerstr. 22a, 7000 Stuttgart 1, Germany
    :Copyright.  Public Domain, (C) Copyright 1988 by Michael Frie
    :Language.   Modula-2
    :Translator. M2Amiga A+L V3.2d
    :Imports.    TaskMemory [bne]
    :History.    V1.0 [mif] 14.Sep.1988
    :History.    V1.1 [bne] 5.Feb.1989 (+ QueueAllocProc, Init() changed)
    :History.    complete new implementation
    :History.    V1.2 [bne] 5.Feb.1989 (+ Get, Remove)

**********************************************************************)

IMPLEMENTATION MODULE Queue;

FROM SYSTEM     IMPORT BYTE, ADR, ADDRESS;
FROM TaskMemory IMPORT Allocate, Deallocate;
FROM Exec       IMPORT CopyMem,MinList,MinListPtr,MinNode,MinNodePtr,
                AddTail,RemHead;

CONST   copyright = "MODULE Queue: (C) Copyright 1988 by Michael Frie";

TYPE    queue      = MinListPtr;
        ElementPtr = POINTER TO Element;
        Element    = RECORD
                       node:MinNode;
                       Data:(* ARRAY OF *)BYTE;
                     END;

CONST   Offset=SIZE(MinNode);

PROCEDURE Init(VAR q:queue):BOOLEAN;
BEGIN
  QueueAllocProc(q,SIZE(MinList));
  IF q#NIL THEN
    WITH q^ DO  (* NewList() *)
      head:=ADR(tail);
      tail:=NIL;
      tailPred:=ADDRESS(q);
    END;
    RETURN TRUE;
  ELSE
    RETURN FALSE;
  END;
END Init;

PROCEDURE Write(q:queue;Data:ARRAY OF BYTE):BOOLEAN;
VAR     Bottom:ElementPtr;
        DataSize:LONGINT;
BEGIN
  DataSize:=HIGH(Data)+1;
  QueueAllocProc(Bottom,Offset+DataSize);
  IF Bottom#NIL THEN
    CopyMem(ADR(Data),ADR(Bottom^.Data),DataSize);
    AddTail(ADDRESS(q),Bottom);
    RETURN TRUE;
  ELSE
    RETURN FALSE;
  END;
END Write;

PROCEDURE Read (q:queue;VAR Data:ARRAY OF BYTE);
VAR     Top:ElementPtr;
BEGIN
  Top:=RemHead(ADDRESS(q));
  CopyMem(ADR(Top^.Data),ADR(Data),HIGH(Data)+1);
  QueueDeallocProc(Top);
END Read;

PROCEDURE Get (q:queue; VAR DataPtr:ADDRESS);
BEGIN
  DataPtr:=LONGINT(q^.head)+Offset;
END Get;

PROCEDURE Remove (q:queue);
VAR     Node:ElementPtr;
BEGIN
  Node:=RemHead(ADDRESS(q));
  QueueDeallocProc(Node);
END Remove;

PROCEDURE Empty(q:queue):BOOLEAN;
BEGIN
  RETURN q^.head^.succ = NIL
END Empty;

PROCEDURE Discard(VAR q:queue);
BEGIN
  WHILE NOT Empty(q) DO
    Remove(q);
  END;
  QueueDeallocProc(q);
END Discard;

BEGIN
  QueueAllocProc:=Allocate;
  QueueDeallocProc:=Deallocate;
END Queue.
