(*---------------------------------------------------------------------------
  :Program.    ILBMHandler.mod
  :Author.     Stefan Salewski
  :Address.    Stolper Weg 3, D-2160 Stade
  :Shortcut.   [sts]
  :Version.    1.0
  :Date.       2-apr-89
  :Copyright.  Public Domain
  :Language.   Modula-2
  :Translator. M2Amiga AMSoft V3.2d
  :Contents.   Procedures for loading and saving ILBM-Pictures
---------------------------------------------------------------------------*)

IMPLEMENTATION MODULE ILBMHandler;
(*
  Created:   1/89
  Changed:   14.2.88/23.2.89/25.2.89/10.3.89/14.3.89/17.3.89/29.3.89 by
             Stefan Salewski
             Stolper Weg 3
             2160 Stade   West-Germany
             Tel: 04141/61130
             
  Note: compiled with AMIGA Modula-2 System, 3.2d, (1.11.88) by AMSoft
   
  This Module may be freely copied. But please
  leave our names in. Thanks....Stefan
  
  
  This Module is base on the Module
  IFFSupport (Amok Disk 6) by Fridtjof Siebert (Amok Computer-Club)
                              Nobileweg 67
                              D-7000 Stuttgart 40
                  
*)
  FROM SYSTEM    IMPORT ADR,ADDRESS,SHIFT,BITSET,LONGSET,CAST,INLINE,REG,BYTE,
                        SETREG;
  FROM Dos       IMPORT Delay;
  FROM Arts      IMPORT TermProcedure,Assert,CurrentLevel;
  FROM Exec      IMPORT AllocMem,FreeMem,MemReqs,MemReqSet,CopyMem,UByte,Byte,
                        Interrupt,AddIntServer,RemIntServer,NodeType;
  FROM Intuition IMPORT NewScreen,ScreenPtr,ScreenFlags,ScreenFlagSet,
                        customScreen,IDCMPFlags,IDCMPFlagSet,DisplayBeep;
  IMPORT Graphics;
  FROM Graphics  IMPORT SetRGB4,RastPort,RastPortPtr,BitMap,BitMapPtr,
                        ViewModes,ViewModeSet,InitBitMap,AllocRaster,
                        FreeRaster,ViewPortPtr,RectanglePtr,GfxBasePtr,
                        Rectangle,GetRGB4,BltBitMapRastPort;
  FROM Hardware  IMPORT vertb;
  IMPORT FileSystem;
  (* FileSystem hat gegenueber DOS den Vorteil, dass die Schreib- und
     Leseoperationen gepuffert sind. Dadurch werden diese Operationen
     wesentlich beschleinigt. Asserden schliesst FileSystem bei
     einen Programmabbruch alle Files.
  *)
  
  CONST
    LoResStandWidth	=320;
    LoResOverscanWidth	=704 DIV 2;
    NTSCNonIntStandH	=200;
    PalNonIntStandH	=256;
    NTSCNonIntOverH	=262; (* oder 464 DIV 2 oder 452 DIV 2 ??? *)
    PalNonIntOverH	=290;(*572 DIV 2*)
    
    MaxImageWidth=2048; (* momentan ist wohl 1024 das maximum ? *)
    FullSpeed=4000H;
    MOVEMS=48E7H; (* that's the 68000-Instruction MOVEM to save Registers*)
    MOVEML=4CDFH; (* that's MOVEM to load Registers                      *)
    BMHDSize=28;
    GRABSize=12;
    DESTSize=16;
    CAMGSize=12;
    CRNGSize=16;
    Buffer0Size=(MaxImageWidth DIV 8)*2; (* 512 Bytes*)
    Buffer1Size=Buffer0Size;
    BufferSize=Buffer0Size+Buffer1Size;
    
  TYPE
    BytePtr=POINTER TO Byte;
    UBytePtr=POINTER TO UByte;
    BitSetPtr=POINTER TO BITSET;
    IFFInfoPtr=POINTER TO IFFInfo;
        
  VAR
    myLevel:INTEGER;
    inFile,outFile:FileSystem.File;
    cyc1,cyc2,cyc3:UByte;
    buffer:ADDRESS;              (* Buffer for Read,Write,Shift              *)
    buffer1:ADDRESS;             (* ADR(buffer)+512                          *)
    datenAdr:ADDRESS;            (* momentaner Ort der Daten                 *)
    len:LONGINT;                 (* Receives Length from Read/Write()        *)
    infoPtr:IFFInfoPtr;          (* Interrupt's IFFInfo                      *)
    defaultRect:Rectangle;
    formName,ilbmName,bmhdName,cmapName,camgName:fourChar;
    grabName,destName,crngName,bodyName:fourChar;
    null:UByte;
    integer:INTEGER; (* temporaere Hilfsvariablen *)
    cardinal:CARDINAL;
    longint:LONGINT;
    
(**********************************************************************)
  
  PROCEDURE Equal(VAR s1,s2:fourChar):BOOLEAN;
  BEGIN
    RETURN ((s1[0]=s2[0]) AND (s1[1]=s2[1]) AND (s1[2]=s2[2]) AND (s1[3]=s2[3]))
  END Equal;
  
(**********************************************************************)

  PROCEDURE MinInt(i,j:LONGINT):LONGINT;
  BEGIN
    IF i<j THEN
      RETURN i
    ELSE
      RETURN j
    END
  END MinInt;
  
(**********************************************************************)
  
  PROCEDURE MaxInt(i,j:LONGINT):LONGINT;
  BEGIN
    IF i>j THEN
      RETURN i
    ELSE
      RETURN j
    END
  END MaxInt;

(**********************************************************************)

  PROCEDURE ScreenSize(VAR width,height:INTEGER;overscan:BOOLEAN);
    VAR
      gfxBasePtr:GfxBasePtr;
  BEGIN
    gfxBasePtr:=ADR(Graphics);
    IF overscan THEN
      width:=LoResOverscanWidth;
      IF Graphics.ntsc IN gfxBasePtr^.displayFlags THEN
        height:=NTSCNonIntOverH
      ELSE
        height:=PalNonIntOverH
      END
    ELSE
      width:=LoResStandWidth;
      IF Graphics.ntsc IN gfxBasePtr^.displayFlags THEN
        height:=NTSCNonIntStandH
      ELSE
        height:=PalNonIntStandH
      END
    END
  END ScreenSize;
   
(**********************************************************************)   
  
  PROCEDURE UpperWord(i:INTEGER):INTEGER;
  BEGIN
    RETURN (i+15)-((i+15) MOD 16)
  END UpperWord;

(**********************************************************************)
  
  PROCEDURE LowerWord(i:INTEGER):INTEGER;
  BEGIN
    RETURN i-(i MOD 16)
  END LowerWord;
  
(**********************************************************************)
  
  PROCEDURE InitScreen(VAR nS:NewScreen;VAR info:IFFInfo;overscan:BOOLEAN);
    VAR w,h:INTEGER;
  BEGIN
    ScreenSize(w,h,overscan);
    nS.width:=MaxInt(info.bmhd.width,info.bmhd.scrnWidth);
    nS.height:=MaxInt(info.bmhd.height,info.bmhd.scrnHeight);
    nS.leftEdge:=LowerWord(info.bmhd.left);(* currently leftEdge=0 *)
    nS.topEdge:=info.bmhd.top;
    nS.depth:=info.bmhd.depth;
    nS.viewModes:=ViewModeSet{};
    IF (nS.width>w) AND (nS.depth<5) THEN INCL(nS.viewModes,hires) END;
    IF nS.height>h THEN INCL(nS.viewModes,lace) END;
    IF camg IN info.IFFTitle THEN
      WITH info.camg DO
        IF (Lace  IN viewType) THEN INCL(nS.viewModes,lace  ) END;
        IF (HoMod IN viewType) THEN INCL(nS.viewModes,ham   ) END;
        IF (Hires IN viewType) THEN INCL(nS.viewModes,hires ) END;
        IF (DblPF IN viewType) THEN INCL(nS.viewModes,dualpf) END;
        IF (Extra IN viewType) THEN INCL(nS.viewModes,extraHalfbrite) END;
      END;
    END;
    nS.detailPen:=0;
    nS.blockPen:=1;
    nS.type:=customScreen+ScreenFlagSet{screenQuiet};
    nS.font:=NIL;
    nS.defaultTitle:=NIL;
    nS.gadgets:=NIL;
    nS.customBitMap:=NIL;
  END InitScreen;
  
  (**********************************************************************)

  PROCEDURE DimOn(screenPtr:ScreenPtr;VAR info:IFFInfo;delay:UByte);
    VAR
      m,n:CARDINAL;
  BEGIN
    IF (delay=0) OR (ham IN screenPtr^.viewPort.modes) THEN
      m:=15
    ELSE
      m:=0;
      DEC(delay);
    END;
    WITH info.cmap DO
      REPEAT
        INC(m);
        FOR n:=0 TO colorCnt-1 DO
          cardinal:=3*n;
          SetRGB4(ADR(screenPtr^.viewPort),n,
                  (colors[cardinal+Red]*m) DIV 16,
                  (colors[cardinal+Green]*m) DIV 16,
                  (colors[cardinal+Blue]*m) DIV 16);
        END;
        IF delay>0 THEN
          Delay(delay)
        END;
      UNTIL m>=16;
    END;
  END DimOn;
  
(**********************************************************************)

  PROCEDURE DimOff(screenPtr:ScreenPtr;VAR info:IFFInfo;delay:UByte);
    VAR m,n:CARDINAL;
  BEGIN
    IF NOT(ham IN screenPtr^.viewPort.modes) THEN
      WITH info.cmap DO
        IF delay=0 THEN
          m:=1
        ELSE
          m:=16;
          DEC(delay);
        END;
        REPEAT
          DEC(m);
          FOR n:=0 TO colorCnt-1 DO
            cardinal:=3*n;
            SetRGB4(ADR(screenPtr^.viewPort),n,
                    (colors[cardinal+Red]*m) DIV 16,
                    (colors[cardinal+Green]*m) DIV 16,
                    (colors[cardinal+Blue]*m) DIV 16);
          END;
          IF delay>0 THEN
            Delay(delay)
          END;
        UNTIL m<=0
      END;
    ELSE
      DimOn(screenPtr,info,0)
    END;
  END DimOff;
  
(**********************************************************************)
  
  PROCEDURE MakeBitMap(width,height:Pixel;depth:Depth):BitMapPtr;
  VAR
    i:Depth;
    error:BOOLEAN;
    bitMapPtr:BitMapPtr;
  BEGIN
    bitMapPtr:=AllocMem(SIZE(BitMap),MemReqSet{public});
    InitBitMap(bitMapPtr^,depth,width,height);
    i:=0;
    REPEAT
      bitMapPtr^.planes[i]:=AllocRaster(width,height);
      error:=bitMapPtr^.planes[i]=NIL;
      INC(i);
    UNTIL (i=depth) OR error;
    IF error THEN
      WHILE i>=1 DO
        DEC(i);
        FreeRaster(bitMapPtr^.planes[i],width,height);
      END;
      FreeMem(bitMapPtr,SIZE(BitMap));
      RETURN NIL;
    ELSE
      RETURN bitMapPtr
    END
  END MakeBitMap;
  
(**********************************************************************)
  
  PROCEDURE FreeBitMap(VAR bmPtr:BitMapPtr);
  VAR
    i:CARDINAL;
  BEGIN
    IF bmPtr#NIL THEN
      FOR i:=0 TO bmPtr^.depth-1 DO
        FreeRaster(bmPtr^.planes[i],bmPtr^.bytesPerRow*8,bmPtr^.rows);
      END;
      FreeMem(bmPtr,SIZE(BitMap));
    END;
    bmPtr:=NIL;
  END FreeBitMap;
  
(**********************************************************************)
  
  PROCEDURE ReadInfo(name:ARRAY OF CHAR;VAR info:IFFInfo;
                     VAR bodyPos:LONGINT):BOOLEAN;
    VAR
      bitSetPtr:POINTER TO BITSET ;
      bitSet:BITSET;
      kennPtr:POINTER TO fourChar;
      buf:ARRAY[0..3] OF fourChar;
      size:LONGINT;
      bodyFound:BOOLEAN;
      uByte:UByte;
    
  BEGIN
    iffError:=noError;
    bodyFound:=FALSE;
    bodyPos:=-1;
    info.IFFTitle:=IFFTitleSet{};
    info.crng.count:=0;
    FileSystem.Lookup(inFile,name,256+128,FALSE);
    IF inFile.res#FileSystem.done THEN
      (* Close(inFile) fuehrt hier zum Guru *)
      iffError:=openFileFailed;
      RETURN FALSE
    END;
    FileSystem.ReadBytes(inFile,ADR(buf),16,len);
    IF NOT((len=16) & Equal(buf[0],formName) & Equal(buf[2],ilbmName)) THEN
      iffError:=noILBM;
    END;
    kennPtr:=ADR(buf[3]);
    WHILE (iffError=noError) AND NOT(bodyFound) DO
      IF Equal(kennPtr^,bmhdName) THEN
        INCL(info.IFFTitle,bmhd);
        FileSystem.ReadBytes(inFile,ADR(info.bmhd),BMHDSize,len);
        kennPtr:=ADR(info.bmhd.nextName);
      ELSIF Equal(kennPtr^,cmapName) THEN
        INCL(info.IFFTitle,cmap);
        FileSystem.ReadBytes(inFile,ADR(info.cmap.size),4,len);
        FileSystem.ReadBytes(inFile,ADR(info.cmap)+4,info.cmap.size+4,len);
        WITH info.cmap DO
          colorCnt:=size DIV 3;
          FOR integer:=0 TO size-1 DO
            colors[integer]:=colors[integer] DIV 16;
          END
        END;
        kennPtr:=ADDRESS(ADR(info.cmap)+LONGINT(info.cmap.size+4));
      ELSIF Equal(kennPtr^,camgName) THEN
        INCL(info.IFFTitle,camg);
        FileSystem.ReadBytes(inFile,ADR(info.camg),CAMGSize,len);
        kennPtr:=ADR(info.camg.nextName);
      ELSIF Equal(kennPtr^,grabName) THEN
        INCL(info.IFFTitle,grab);
        FileSystem.ReadBytes(inFile,ADR(info.grab),GRABSize,len);
        kennPtr:=ADR(info.grab.nextName); 
      ELSIF Equal(kennPtr^,destName) THEN
        INCL(info.IFFTitle,dest);
        FileSystem.ReadBytes(inFile,ADR(info.dest),DESTSize,len);
        kennPtr:=ADR(info.dest.nextName);
      ELSIF Equal(kennPtr^,crngName) THEN
        FileSystem.ReadBytes(inFile,ADR(info.crng.data[info.crng.count]),
                             CRNGSize,len);
        kennPtr:=ADR(info.crng.data[info.crng.count].nextName);
        WITH info.crng.data[info.crng.count] DO
          bitSetPtr:=ADR(on);
          bitSet:=bitSetPtr^;
          on:=(0 IN bitSet);
          forward:=NOT(1 IN bitSet);
          on:=on AND (low<info.cmap.colorCnt)  AND (rate<=FullSpeed)
                 AND (high<info.cmap.colorCnt) AND (rate>0) AND (low#high);
          IF on THEN
            IF low>high THEN
              uByte:=low;
              low:=high;
              high:=uByte;
              forward:=FALSE;
            END;
            INC(info.crng.count);
            INCL(info.IFFTitle,crng);
          END;
        END;
      ELSIF Equal(kennPtr^,bodyName) THEN
        bodyFound:=TRUE;
        FileSystem.GetPos(inFile,bodyPos);
        DEC(bodyPos,4);
      ELSE (* Ignore unknown data *)
        FileSystem.ReadBytes(inFile,ADR(size),4,len);
        WHILE size>BufferSize-4 DO
          FileSystem.ReadBytes(inFile,buffer,BufferSize-4,len);
          DEC(size,BufferSize-4);
        END;
        FileSystem.ReadBytes(inFile,buffer,size+4,len);
        kennPtr:=ADDRESS(buffer+size)
      END;
      IF len=0 THEN
        iffError:=readError;
      END;
    END;
    FileSystem.Close(inFile);
    RETURN bodyFound AND (info.IFFTitle>=IFFTitleSet{bmhd,cmap});
  END ReadInfo;
  
(**********************************************************************)

  PROCEDURE MaskPlane(VAR info:IFFInfo):Depth;
  BEGIN
    IF info.bmhd.masking=1 THEN
      RETURN 1
    ELSE
      RETURN 0
    END
  END MaskPlane;
  
(**********************************************************************)
  
  PROCEDURE ReadBody(name:ARRAY OF CHAR;bitmap:BitMap;bodyPos:LONGINT;
                     imageWidth,imageX,imageY,destX,destY,width,height:Pixel;
                     planesToIgnore:Depth;imageIsCompressed:BOOLEAN):BOOLEAN;
    VAR
      i,j,k:CARDINAL;
      depth:UByte;
      chunkName:fourChar;
      myPlanes:ARRAY[1..MaxPlanes] OF ADDRESS;
      byte:UByte;
      bytes:ARRAY[0..1] OF UByte;
      (* Um nicht immer wieder Einzelne Bytes von FileSystem anfordern zu
         muessen, lese ich immmer mit den Bilddaten den naesten Zaehler mit
         ein; bytes[1] ist dieser Zaehler.
      *)
      d:POINTER TO UByte;
      count:CARDINAL;
      actBuffer:ADDRESS;
      bodyLength:LONGINT;

  BEGIN
    imageWidth:=UpperWord(imageWidth) DIV 8;
    width :=((imageX MOD 8) +width+7) DIV 8;
    imageX:=imageX DIV 8;
    destX :=destX  DIV 8;
    IF (imageX+width>imageWidth) OR
       (*(imageY+height>imageHeight) OR ==> ReadError*)
       (destX+width>bitmap.bytesPerRow) OR
       (destY+height>bitmap.rows) THEN
       iffError:=wrongSize;
       RETURN FALSE
    END;
    iffError:=noError;
    depth:=bitmap.depth;
    FileSystem.Lookup(inFile,name,1024,FALSE);
    IF inFile.res#FileSystem.done THEN
      (* Close(inFile) fuehrt hier zum Guru *)
      iffError:=openFileFailed;
      RETURN FALSE;
    END;
    FileSystem.SetPos(inFile,bodyPos);
    FileSystem.ReadBytes(inFile,ADR(chunkName),4,len);
    IF NOT((len=4) AND Equal(chunkName,bodyName)) THEN
      iffError:=noILBM
    ELSE
      actBuffer:=buffer+LONGINT(imageX);
      FileSystem.ReadBytes(inFile,ADR(bodyLength),4,len);
      longint:=LONGINT(destY*bitmap.bytesPerRow+destX);
      FOR i:=1 TO depth DO
        myPlanes[i]:=bitmap.planes[i-1]+longint;
      END;
      IF imageIsCompressed THEN
        FileSystem.ReadBytes(inFile,ADR(bytes[1]),1,len);
        FOR i:=1 TO imageY DO (* Die obersten imageY Zeilen ueberlesen *)
          FOR j:=1 TO depth+planesToIgnore DO
            d:=buffer;
            count:=0;
            WHILE (count<imageWidth) AND (len#0) DO
              IF bytes[1]<128 THEN
                INC(bytes[1]);
                FileSystem.ReadBytes(inFile,d,bytes[1]+1,len);
                INC(d,bytes[1]);
                INC(count,bytes[1]);
                bytes[1]:=d^;
              ELSE
                byte:=257-bytes[1];
                FileSystem.ReadBytes(inFile,ADR(bytes[0]),2,len);
                INC(count,byte);
              END
            END;
          END;
        END;
        FOR i:=1 TO height DO (* Den Bildausschnitt lesen *)
          FOR j:=1 TO depth+planesToIgnore DO
            d:=buffer;
            count:=0;
            WHILE (count<imageWidth) AND (len#0) DO
              IF bytes[1]<128 THEN
                INC(bytes[1]);
                FileSystem.ReadBytes(inFile,d,bytes[1]+1,len);
                INC(d,bytes[1]);
                INC(count,bytes[1]);
                bytes[1]:=d^;
              ELSE
                byte:=257-bytes[1];
                FileSystem.ReadBytes(inFile,ADR(bytes[0]),2,len);
                (*FOR k:=1 TO byte DO
                  d^:=bytes[0];
                  INC(d);
                END;
                *)
                (*INLINE(048E7H,0C080H);               movem.l d0-d1/a0,-(a7) *)
                (* das Retten der Register d0,d1 und a0 ist wohl unnoetig     *)
                SETREG(0,LONGCARD(bytes[0]));       (* move.l  bytes[0],d0    *)
                SETREG(1,LONGCARD(byte));           (* move.l  byte,d1        *)
                SETREG(8,d);                        (* lea     d,a0           *)
                INLINE(05301H,010C0H,051C9H,0FFFCH);(* subq.b #1,d1           *)
                                                    (* Loop:                  *)
                                                    (* move.b d0,(a0)+        *)
                                                    (* dbra d1,Loop           *)
                (*INLINE(04CDFH,00103H);               movem.l (a7)+,d0-d1/a0 *)
                INC(count,byte);               
                INC(d,byte);                   
              END
            END;
            IF j<=bitmap.depth THEN
              CopyMem(actBuffer,myPlanes[j],width);
              INC(myPlanes[j],bitmap.bytesPerRow);
            END;
          END;
        END;
      ELSE
        FOR i:=1 TO imageY DO
          FOR j:=1 TO depth+planesToIgnore DO
            FileSystem.ReadBytes(inFile,buffer,imageWidth,len);
          END;
        END;
        FOR i:=1 TO height DO
          FOR j:=1 TO depth DO
            FileSystem.ReadBytes(inFile,buffer,imageWidth,len);
            CopyMem(actBuffer,myPlanes[j],width);
            INC(myPlanes[j],bitmap.bytesPerRow);
          END;
          FOR j:=1 TO planesToIgnore DO
            FileSystem.ReadBytes(inFile,buffer,imageWidth,len);
          END;
        END;
      END;
    END;
    IF len=0 THEN
      iffError:=readError;
    END;
    FileSystem.Close(inFile);
    RETURN (iffError=noError);
  END ReadBody;
  
(**********************************************************************)
  
  PROCEDURE LoadInRastPort(name:ARRAY OF CHAR;rPtr:RastPortPtr;bodyPos:LONGINT;
                           imageWidth,imageX,imageY,destX,destY,width,
                           height:INTEGER;planesToIgnore:Depth;
                           imageIsCompressed:BOOLEAN):BOOLEAN;
    VAR
      tmpBMPtr:BitMapPtr;
  BEGIN
    tmpBMPtr:=MakeBitMap(width,height,rPtr^.bitMap^.depth);
    IF tmpBMPtr=NIL THEN
      iffError:=outOfMem;
      RETURN FALSE
    END;
    IF NOT ReadBody(name,tmpBMPtr^,bodyPos,imageWidth,imageX,imageY,
                    0,0,width,height,planesToIgnore,imageIsCompressed) THEN
      RETURN FALSE
    ELSE
      BltBitMapRastPort(tmpBMPtr,imageX MOD 8,0,
                      rPtr,destX,destY,width,height,0C0H);
      FreeBitMap(tmpBMPtr);
      RETURN TRUE
    END;
  END LoadInRastPort;
  
(**********************************************************************)
  
(* $S- *)
(* Die Stackkontrolle muss mit $S- unbedingt ausgeschaltet werden, da
   die Interruptserverprozedur in Supervisormodus laeuft und deshalb 
   den Supervisor-Stack benutzt.
*)
  PROCEDURE CycleInterrupt; (* InterruptServerProcedure *)
  BEGIN
    INLINE(MOVEMS,3F3EH); (* Movem.l D2-D7/A2-A6,+(SP) *)
    infoPtr:=ADDRESS(REG(9));
    FOR cyc1:=0 TO infoPtr^.crng.count-1 DO
      IF infoPtr^.crng.data[cyc1].on THEN
        INC(infoPtr^.cycleInfo.speedCnt[cyc1],infoPtr^.crng.data[cyc1].rate);
        IF infoPtr^.cycleInfo.speedCnt[cyc1]>=FullSpeed THEN
          DEC(infoPtr^.cycleInfo.speedCnt[cyc1],FullSpeed);
          IF infoPtr^.crng.data[cyc1].forward THEN
            IF infoPtr^.cycleInfo.base[cyc1]<=
              infoPtr^.crng.data[cyc1].low THEN
              infoPtr^.cycleInfo.base[cyc1]:=infoPtr^.crng.data[cyc1].high;
            ELSE
              DEC(infoPtr^.cycleInfo.base[cyc1]);
            END;
          ELSE
            IF infoPtr^.cycleInfo.base[cyc1]>=
            infoPtr^.crng.data[cyc1].high THEN
              infoPtr^.cycleInfo.base[cyc1]:=
              infoPtr^.crng.data[cyc1].low;
            ELSE
              INC(infoPtr^.cycleInfo.base[cyc1]);
            END;
          END;
          cyc3:=infoPtr^.cycleInfo.base[cyc1];
          FOR cyc2:=infoPtr^.crng.data[cyc1].low TO 
                    infoPtr^.crng.data[cyc1].high DO
                    cardinal:=3*cyc3;
            SetRGB4(infoPtr^.cycleInfo.vpPtr,
                    cyc2,infoPtr^.cmap.colors[cardinal+Red],
                    infoPtr^.cmap.colors[cardinal+Green],
                    infoPtr^.cmap.colors[cardinal+Blue]);
            INC(cyc3);
            IF cyc3>infoPtr^.crng.data[cyc1].high THEN
              cyc3:=infoPtr^.crng.data[cyc1].low
            END;
          END;
        END;
      END;
    END;
    INLINE(MOVEML,7CFCH); (* Movem.l (SP)-,D2-D7/A2-A6 *)
  END CycleInterrupt;
(* $S= *)

(***************************************************************************)

  PROCEDURE StartCycling(VAR info:IFFInfo;screenPtr:ScreenPtr):BOOLEAN;
    VAR j:INTEGER;
  BEGIN
    IF (crng IN info.IFFTitle) AND (screenPtr#NIL) THEN
      info.cycleInfo.vpPtr:=ADR(screenPtr^.viewPort);
      FOR cardinal:=0 TO info.crng.count-1 DO
        info.cycleInfo.base[cardinal]:=info.crng.data[cardinal].low;
        info.cycleInfo.speedCnt[cardinal]:=0;
      END;
      WITH info.cycleInfo.int DO
        node.type:=interrupt;
        node.pri:=-60;
        node.name:=NIL;
        data:=ADR(info);
        code:=CycleInterrupt;
      END;
      SETREG(0,0); (* Nur bei V3.2 noetig, Compilerfehler !!! *)
      AddIntServer(vertb,ADR(info.cycleInfo.int));
      RETURN TRUE;
    ELSE
      RETURN FALSE
    END
  END StartCycling;

(***************************************************************************)

  PROCEDURE EndCycling(VAR info:IFFInfo);
  BEGIN
    Assert(info.cycleInfo.vpPtr#NIL,ADR('EndCycling without StartCycling!'));
    SETREG(0,0);(* Nur bei V3.2 noetig, Compilerfehler !!! *)
    RemIntServer(vertb,ADR(info.cycleInfo.int));
  END EndCycling;
  
(***************************************************************************)

  PROCEDURE InitIFFInfo(VAR info:IFFInfo;
                        rpPtr:RastPortPtr;
                        vpPtr:ViewPortPtr);
    VAR
      colorConv:LONGCARD;
      i:LONGINT;
  BEGIN
    WITH info.bmhd DO
      (* 
      size:=BMHDSize-8; \
      width              | wird von SaveBitMap initialisiert
      height            /
      *)
      left:=0;
      top:=0;
      depth:=rpPtr^.bitMap^.depth;
      masking:=0;
      (* compression wird von SaveBitMap initialisiert *)
      pad:=0;
      transCol:=0;
      scrnWidth:=rpPtr^.bitMap^.bytesPerRow*8;
      scrnHeight:=rpPtr^.bitMap^.rows;
      IF hires IN vpPtr^.modes THEN
        xAspect:=5;
      ELSE
        xAspect:=10;
      END;
      IF lace IN vpPtr^.modes THEN
        xAspect:=xAspect*2;
      END;
      yAspect:=11;
    END;
    WITH info.cmap DO
      colorCnt:=vpPtr^.colorMap^.count;
      (* size:=colorCnt*3; wird von SaveBitMap initialisiert *)
      FOR i:=0 TO (colorCnt-1)*3 BY 3 DO
        colorConv:=GetRGB4(vpPtr^.colorMap,i DIV 3);
        colors[i+Red]  :=(colorConv DIV 256) MOD 16;
        colors[i+Green]:=(colorConv DIV 16 ) MOD 16;
        colors[i+Blue] := colorConv MOD 16;
      END;
    END;
    WITH info.camg DO
      (* size:=CAMGSize-8; wird von SaveBitMap initialisiert *)
      viewType:=ViewTypeSet{};
      IF lace           IN vpPtr^.modes THEN INCL(viewType,Lace)  END;
      IF hires          IN vpPtr^.modes THEN INCL(viewType,Hires) END;
      IF dualpf         IN vpPtr^.modes THEN INCL(viewType,DblPF) END;
      IF ham            IN vpPtr^.modes THEN INCL(viewType,HoMod) END;
      IF extraHalfbrite IN vpPtr^.modes THEN INCL(viewType,Extra) END;
    END;
    info.IFFTitle:=IFFTitleSet{bmhd,cmap,camg};
  END InitIFFInfo;
  
(***********************************************************************)

  PROCEDURE SaveBitMap(name:ARRAY OF CHAR;
                       VAR info:IFFInfo;
                       bmPtr:BitMapPtr;
                       rectPtr:RectanglePtr;
                       compressIt:BOOLEAN):BOOLEAN;

    VAR
      myPlanes:ARRAY[0..MaxPlanes-1] OF ADDRESS;
      bytesPerImageLine,wordsPerImageLine:INTEGER;
      fileSize,bodySizePos,bodySize:LONGINT;
      needToShift:BOOLEAN;
      bitsToShift:[0..15];
      lowerLeft,upperWidth:INTEGER;(* Word-aligned Offset & Width *)
      bitSetPtr:BitSetPtr;
      bitSet:BITSET;
      rMask,lMask:LONGSET;
      newLength:INTEGER;
      i,j:INTEGER;
    
    PROCEDURE Compress(source,dest:ADDRESS;
                       sourceLength:INTEGER;VAR destLength:INTEGER);
(*  Die sourceLength Bytes, die an der Position source stehen,
    werden komprimiert und an der Speicherstelle dest abgespeichert.
    destLength ist die Laenge dieser kompriemierten Daten in Bytes.
    Die komprimierten Daten sind folgendermassen zu interpretieren:
    Ist ein Byte
    0..127 :  bedeutet das: Die naechsten n+1 (1..128) Bytes
              unveraendert uebernehmen.
    129..255: Das folgende Byte wird 257-n mal dubliziert.
    128:      keine Bedeutung.
   
    Der Speicher dest muss mindestens 1.5*sourceLength gross sein,
    denn die Komprimierung kann theoretisch bei sehr unguenstigen Daten
    auch negativ ausfallen!
   
*)    CONST
        PackLimit=2; (* Wenn mehr als PackLimit Bytes gleich sind,
                        so wird gepackt. PackLimit=2 ist Standart
                        und wohl auch optimal, die Prozedur funktioniert
                        aber fuer Werte von 1 bis 128 *)
      VAR
        pos,byte1,byte2:UBytePtr;
        count:BytePtr;
        gleiche,i:UByte;
        ungleicheGleichNull:BOOLEAN;
        len:INTEGER;
    BEGIN
      len:=0;
      ungleicheGleichNull:=TRUE;
      pos:=dest;
      byte1:=source;
      DEC(byte1);
      byte2:=source;
      LOOP
        gleiche:=0;
        REPEAT
          INC(gleiche);
          INC(byte1);INC(byte2);INC(len);
        UNTIL (byte2^#byte1^) OR (len>=sourceLength) OR (gleiche>=128);
        IF gleiche>PackLimit THEN (* wir koennen packen *)
          pos^:=257-gleiche;
          INC(pos);
          pos^:=byte1^;
          INC(pos);
          ungleicheGleichNull:=TRUE;
        ELSIF len<=sourceLength THEN (* zuwenig gleiche Bytes gefunden *)
          IF ungleicheGleichNull OR (count^>=128-PackLimit) THEN
            ungleicheGleichNull:=FALSE;
            count:=ADDRESS(pos);
            count^:=-1;
            INC(pos);
          END;
          FOR i:=1 TO gleiche DO
            INC(count^);
            pos^:=byte1^;
            INC(pos);
          END;
        ELSE
          EXIT
        END;
      END;
      destLength:=LONGINT(pos)-dest;
    END Compress;

    PROCEDURE Shift(from,to:ADDRESS;words,count:INTEGER);
    (* Kopiert 2*words bytes von der Speicherstelle from nach to,
       wobei diese Bytes gleichzeitig um count Bits nach links verschoben
       werden. Es muss gelten: 0<= count <=15 .
    *)
      VAR
        lwPtr,toPtr:POINTER TO LONGSET;
        h:LONGSET;
        i:INTEGER;
    BEGIN
      toPtr:=to;
      IF count=0 THEN
        CopyMem(from,to,words*2);
        INC(toPtr,(words-2)*2)
      ELSE
        lwPtr:=from;
        toPtr^:=CAST(LONGSET,SHIFT(CAST(LONGCARD,lwPtr^),count));
        FOR i:=0 TO words-2 DO
          INC(lwPtr,2);
          INC(toPtr,2);
          h:=CAST(LONGSET,SHIFT(CAST(LONGCARD,lwPtr^),count));
          h:=h-lMask;
          toPtr^:=toPtr^*lMask;
          toPtr^:=h+toPtr^;
        END;
        DEC(toPtr,2);
      END;
      toPtr^:=toPtr^-rMask;
    END Shift;
    
  BEGIN (* SaveBitMap *)
    IF NOT(info.IFFTitle>=IFFTitleSet{bmhd,cmap}) THEN
      iffError:=noILBM;
      RETURN FALSE
    END;
    IF rectPtr=NIL THEN
      rectPtr:=ADR(defaultRect);
      WITH rectPtr^ DO
        minX:=0;
        minY:=0;
        maxX:=bmPtr^.bytesPerRow*8-1;
        maxY:=bmPtr^.rows-1;
      END;
    END;
    info.bmhd.width:=rectPtr^.maxX-rectPtr^.minX+1;
    info.bmhd.height:=rectPtr^.maxY-rectPtr^.minY+1;
    lowerLeft:=LowerWord(rectPtr^.minX);
    upperWidth:=UpperWord(info.bmhd.width);
    longint:=rectPtr^.minY*INTEGER(bmPtr^.bytesPerRow)+(lowerLeft DIV 8);
    FOR i:=0 TO bmPtr^.depth-1 DO
      myPlanes[i]:=bmPtr^.planes[i]+longint;
    END;
    IF compressIt THEN
      info.bmhd.compression:=1
    ELSE
      info.bmhd.compression:=0
    END;
    FileSystem.Lookup(outFile,name,1024,TRUE);
    IF outFile.res#FileSystem.done THEN
      iffError:=openFileFailed;
      RETURN FALSE;
    END;
    FileSystem.WriteBytes(outFile,ADR(formName),8,len);
    (* formName ist nur 4 Bytes gross,der Rest ist beliebig *)
    FileSystem.WriteBytes(outFile,ADR(ilbmName),4,len);
    info.bmhd.size:=BMHDSize-8;
    FileSystem.WriteBytes(outFile,ADR(bmhdName),4,len);
    FileSystem.WriteBytes(outFile,ADR(info.bmhd),BMHDSize-4,len);
    (* IF len # BMHDSize-4 THEN RETURN FALSE ...*)
    (* Ich gehe davon aus, da wenn ein Write fehlschlgt, auch alle
       folgenden Writes nicht erfolgreich sind. Also wird der Fehler
       spter bemerkt.
    *)
    info.cmap.size:=info.cmap.colorCnt*3;
    FOR i:=0 TO MinInt(info.cmap.size-1,3*MaxColors-1) DO
      info.cmap.colors[i]:=info.cmap.colors[i]*16
    END;
    FileSystem.WriteBytes(outFile,ADR(cmapName),4,len);
    FileSystem.WriteBytes(outFile,ADR(info.cmap),info.cmap.size+4,len);
    IF grab IN info.IFFTitle THEN
      info.grab.size:=GRABSize-8;
      FileSystem.WriteBytes(outFile,ADR(grabName),4,len);
      FileSystem.WriteBytes(outFile,ADR(info.grab),GRABSize-4,len);
    END;
    IF dest IN info.IFFTitle THEN
      info.dest.size:=DESTSize-8;
      FileSystem.WriteBytes(outFile,ADR(destName),4,len);
      FileSystem.WriteBytes(outFile,ADR(info.dest),DESTSize-4,len);
    END;
    IF camg IN info.IFFTitle THEN
      info.camg.size:=CAMGSize-8;
      FileSystem.WriteBytes(outFile,ADR(camgName),4,len);
      FileSystem.WriteBytes(outFile,ADR(info.camg),CAMGSize-4,len);
    END;
    IF crng IN info.IFFTitle THEN
      WITH info.crng DO
        FOR i:=0 TO MinInt(count-1,15) DO
          data[i].size:=CRNGSize-8;
          data[i].pad:=0;
          bitSetPtr:=ADR(data[i].on);
          IF data[i].on AND data[i].forward THEN
            bitSetPtr^:=BITSET{0,1}
          ELSIF data[i].on THEN
            bitSetPtr^:=BITSET{0}
          ELSE
            bitSetPtr^:=BITSET{}
          END;
          FileSystem.WriteBytes(outFile,ADR(crngName),4,len);
          FileSystem.WriteBytes(outFile,ADR(data[i]),CRNGSize-4,len);
        END;
      END;
    END;
    (* WriteBody, body-size ist noch unbekannt *)
    FileSystem.WriteBytes(outFile,ADR(bodyName),8,len);
    FileSystem.Length(outFile,bodySizePos);
    DEC(bodySizePos,4);
    bytesPerImageLine:=upperWidth DIV 8; (* Laenge einer Zeile in Bytes *)
    wordsPerImageLine:=bytesPerImageLine DIV 2;
    needToShift:=(lowerLeft#rectPtr^.minX) OR (upperWidth#info.bmhd.width);
  (* Wenn wir einen Ausschnitt aus der Grafik (BitMap) speichern wollen,
     der n*16+x Pixel von linken Rand der BitMap entfernt ist, so
     so lesen wir die Daten ab der Position n*16 aus der BitMap und
     verschieben dann alle Bits einer Zeile um x nach links.
     Dabei sind n und x positive ganze Zahlen. ( n*16+x=rectPtr.minX )
     
     Wenn nur (upperWidth#info.bmhd.width), dann wird zwar nicht geschoben,
     aber wir fuellen jede Zeile bis auf Wordgroesse auf. Die Prozedur
     Shift macht dann eine UND-Verknuepfung mit rMask und dem letzten
     Langwort der Zeile. Dadurch werden die Auffuellbits auf Null gesetzt
  *)
    IF needToShift THEN
      bitsToShift:=rectPtr^.minX-lowerLeft; (* 0<=bitsToShift<16 *)
      lMask:=LONGSET{}; (* Maske die die Prozedur Shift braucht *)
      FOR i:=31 TO 16+bitsToShift BY -1 DO
        INCL(lMask,i)
      END;
      rMask:=LONGSET{}; (* Maske die die Prozedur Shift braucht *)
      FOR i:=0 TO upperWidth-info.bmhd.width-1 DO
        INCL(rMask,i)
      END;
    END;
    FOR i:=0 TO info.bmhd.height-1 DO
      FOR j:=0 TO info.bmhd.depth-1 DO
        (* datenAdr gibt immer den Ort an, wo die zu speichernden
           Daten augenblicklich stehen *)
        IF needToShift THEN
          Shift(myPlanes[j],buffer1,wordsPerImageLine,
                bitsToShift);
          datenAdr:=buffer1;
        ELSE
          datenAdr:=myPlanes[j];
        END;
        IF compressIt THEN
          Compress(datenAdr,buffer,bytesPerImageLine,newLength);
          datenAdr:=buffer;
        ELSE
          newLength:=bytesPerImageLine
        END;
        FileSystem.WriteBytes(outFile,datenAdr,newLength,len);
        INC(myPlanes[j],bmPtr^.bytesPerRow);
      END;
    END;
    FileSystem.Length(outFile,fileSize);
    bodySize:=fileSize-bodySizePos-4;
    IF ODD(fileSize) THEN
      FileSystem.WriteBytes(outFile,ADR(null),1,len);
      INC(fileSize);
    END;
    FileSystem.SetPos(outFile,bodySizePos);
    FileSystem.WriteBytes(outFile,ADR(bodySize),4,len);
    FileSystem.SetPos(outFile,4);
    DEC(fileSize,8);
    FileSystem.WriteBytes(outFile,ADR(fileSize),4,len);
    IF len#4 THEN
      FileSystem.Delete(outFile);
      FileSystem.Close(outFile);
      iffError:=writeError;
      RETURN FALSE;
    ELSE
      FileSystem.Close(outFile);
      RETURN TRUE;
    END;
  END SaveBitMap;
  
(**********************************************************************)

  PROCEDURE SaveRastPort(name:ARRAY OF CHAR;
                         rpPtr:RastPortPtr;
                         vpPtr:ViewPortPtr;
                         rectPtr:RectanglePtr;
                         compressIt:BOOLEAN):BOOLEAN;

  VAR
    iffInfo:IFFInfo;
  BEGIN
    InitIFFInfo(iffInfo,rpPtr,vpPtr);
    RETURN SaveBitMap(name,iffInfo,rpPtr^.bitMap,
                      rectPtr,compressIt);
  END SaveRastPort;
  
(**********************************************************************)

  PROCEDURE SaveScreen(name:ARRAY OF CHAR;
                       screenPtr:ScreenPtr;
                       rectPtr:RectanglePtr;
                       compressIt:BOOLEAN):BOOLEAN;
                            
  BEGIN
    RETURN SaveRastPort(name,ADR(screenPtr^.rastPort),
                        ADR(screenPtr^.viewPort),rectPtr,compressIt);
  END SaveScreen;
  
(**********************************************************************)

  PROCEDURE CleanUp;
  BEGIN
    IF myLevel>=CurrentLevel() THEN
      FreeMem(buffer,BufferSize)
    END
  END CleanUp;
  
BEGIN
  myLevel:=CurrentLevel();
  buffer:=AllocMem(BufferSize,MemReqSet{public,memClear});
  Assert(buffer#NIL,ADR("Sorry, but I need more RAM"));
  buffer1:=buffer+Buffer0Size;
  formName:="FORM";
  ilbmName:="ILBM";
  bmhdName:="BMHD";
  cmapName:="CMAP";
  camgName:="CAMG";
  grabName:="GRAB";
  destName:="DEST";
  crngName:="CRNG";
  bodyName:="BODY";
  null:=0;
  TermProcedure(CleanUp);
END ILBMHandler.
