(*---------------------------------------------------------------------------
  :Program.    MegaWB.mod
  :Author.     Fridtjof Siebert
  :Address.    Nobileweg 67, D-7-Stgt-40
  :Phone.      (0)711/822509
  :Shortcut.   [fbs]
  :Version.    1.2
  :Date.       17-Mar-89
  :Copyright.  PD
  :Language.   Modula-II
  :Translator. M2Amiga v3.1d
  :Contents.   Program to create a 1024 x 512 pixels large Workbench!
---------------------------------------------------------------------------*)

MODULE MegaWB;

FROM SYSTEM      IMPORT ADR, ADDRESS, LONGSET, CAST, BITSET, INLINE, SETREG;
FROM Arts        IMPORT Assert, TermProcedure, Terminate, wbStarted;
FROM Arguments   IMPORT NumArgs, GetArg;
FROM Conversions IMPORT StrToVal;
FROM Exec        IMPORT Forbid, Permit, FindPort, MsgPortPtr, NodeType,
                        Message, MessagePtr, GetMsg, ReplyMsg, PutMsg,
                        WaitPort, IOStdReq, Interrupt, IOStdReqPtr, DoIO,
                        OpenDevice, CloseDevice, AddIntServer, RemIntServer,
                        AllocSignal, FreeSignal, Wait, Signal, FindTask,
                        TaskPtr, Byte, FreeMem, MemReqs, MemReqSet,
                        SetTaskPri, SetFunction;
FROM ExecSupport IMPORT CreatePort, DeletePort, CreateStdIO, DeleteStdIO;
FROM Graphics    IMPORT BitMap, BltBitMap, SimpleSprite, GetSprite, SetRGB4,
                        ChangeSprite, MoveSprite, FreeSprite, WaitBlit,
                        GfxBasePtr, ViewModes, LayerPtr, ViewPortPtr,
                        SimpleSpritePtr, ScrollVPort, UCopListPtr, CWait,
                        CMove, CBump, FreeCopList;
FROM Hardware    IMPORT vertb, custom;
FROM Heap        IMPORT AllocMem;
FROM Icon        IMPORT GetDiskObject, FreeDiskObject, FindToolType,
                        MatchToolValue;
FROM Input       IMPORT inputName, addHandler, remHandler;
FROM InputEvent  IMPORT InputEvent, InputEventPtr, Class, lButton, lAlt,
                        lCommand, rCommand;
FROM Intuition   IMPORT ScreenPtr, MakeScreen, RethinkDisplay, NewWindow,
                        WindowFlags, WindowFlagSet, ScreenFlags, CloseWindow,
                        ScreenFlagSet, IDCMPFlagSet, OpenWindow, WindowPtr,
                        IntuitionBase, GetPrefs, Preferences, GadgetPtr,
                        sizing, wDragging, GadgetFlags;
FROM Layers      IMPORT WhichLayer;
FROM Workbench   IMPORT DiskObjectPtr;
IMPORT Exec, Intuition, Graphics;

(*------  CONSTS:  ------*)

CONST
  WindowTitle = "MegaWB  1989 Fridtjof Siebert / AMOK Stuttgart";
  PortName    = "NewWBPlanes[fbs].Port";
  ReplyName   = "NewWBPlanes[fbs].ReplyPort";
  Usage       = "Usage: MegaWB [p] [width height]";
  TooSmall    = "Can't shrink Workbenchscreen";
  tTWidth     = "WIDTH";
  tTHeight    = "HEIGHT";
  tTProp      = "PROP";
  tTFlags     = "FLAGS";
  tTErr       = "Error in Tooltypes";
  oom         = "Out of memory!";
  noSprites   = "No more free Sprites!";
  noSignal    = "No free Signalbit";
  intName     = "MegaWB.interrupt";
  wins        = "U must move Windows to old WB-Area!";
  MOVEMS = 48E7H;
  MOVEML = 4CDFH;

(*------  TYPES:  ------*)

TYPE
  SprData = POINTER TO ARRAY[0..255] OF LONGINT;

  CantQuit = (cqok,cqoom,cqwin);
  MyMessage = RECORD
                msg: Message;
                cq: CantQuit;
              END;
  MyMessagePtr = POINTER TO MyMessage;


(*------  VARS:  ------*)

VAR
  WBS: ScreenPtr;
  Window: WindowPtr;
  NuWindow: NewWindow;
  MyMsg: MyMessage;
  QuitMessage: MyMessagePtr;
  MyPort, OldPort: MsgPortPtr;
  l: LONGINT;
  bm,oldbm: BitMap;
  iB: POINTER TO IntuitionBase;
  GetSpr,EmptySprite: SprData;
  SprImg: ARRAY[0..1] OF SprData;
  ActSprImg: INTEGER;
  sprite: SimpleSprite;
  sprID: INTEGER;
  oldmx,oldmy: INTEGER;
  lx,ly,mx,my,i,clickX,clickY,cx,cy,rox,roy: INTEGER;
  InputDevPort: MsgPortPtr;
  InputRequestBlock: IOStdReqPtr;
  HandlerStuff: Interrupt;
  HandlerActive, InputOpen: BOOLEAN;
  VertBIntr: Interrupt;
  IntActive: BOOLEAN;
  LMBSig: INTEGER;
  Me: TaskPtr;
  mxm,mym,newmxm,newmym: INTEGER;
  pref: Preferences;
  oldw,oldh: INTEGER;
  w: WindowPtr;
  SizeX,SizeY: LONGINT;
  arg: ARRAY[0..79] OF CHAR;
  err,b: BOOLEAN;
  ev: InputEventPtr;
  gfx: GfxBasePtr;
  lay: LayerPtr;
  lw: WindowPtr;
  gdg: GadgetPtr;
  Sigs: LONGSET;
  gx,gy,gw,gh: INTEGER;
  blitted: BOOLEAN;
  OldMoveSprite: PROCEDURE();
  sx,sy: INTEGER;
  Proportional: BOOLEAN;
  tT: POINTER TO ARRAY [0..79] OF CHAR;
  MyIcon: DiskObjectPtr;
  cl,altcl: UCopListPtr;

(*------  InputHandler:  ------*)

PROCEDURE MyHandler(Ev{8}: InputEventPtr): InputEventPtr; (* $S- *)

BEGIN
  ev := Ev;
  IF (iB^.activeScreen=WBS) AND (WBS^.mouseY>=0) THEN
    WITH WBS^ DO
      sx := mouseX; sy := mouseY;
      cx := 0; cy := 0;
      WHILE ev#NIL DO
        WITH ev^ DO
          IF class=rawmouse THEN
            INC(cx,x); INC(cy,y);
            INC(sx,x); INC(sy,y);
            IF sx-rox>oldw THEN
              rox := sx - oldw;
              IF rox+oldw>SizeX THEN rox := SizeX-oldw END;
            END;
            IF sy-roy>oldh THEN
              roy := sy - oldh;
              IF roy+oldh>SizeY THEN roy := SizeY-oldh END;
            END;
            IF sx-rox<0 THEN
              rox := sx;
              IF rox<0 THEN rox := 0 END;
            END;
            IF sy-roy<0 THEN
              roy := sy;
              IF roy<0 THEN roy := 0 END;
            END;
          END;
          IF (class=rawmouse) AND (code=lButton+128) OR (class=rawkey) AND (code>=128) THEN (* LMB released *)
            newmxm := width - 1; newmym := height + topEdge - 1;
            IF lace  IN viewPort.modes THEN mym := newmym ELSE mym := 2*newmym END;
            IF hires IN viewPort.modes THEN mxm := newmxm ELSE mxm := 2*newmxm END;
          ELSIF (class=rawmouse) AND (code=lButton) OR (class=rawkey) AND (code<128) AND (lAlt IN qualifier) AND ((lCommand IN qualifier) OR (rCommand IN qualifier)) THEN (* LMB pressed *)
            clickX := mouseX; clickY := mouseY;
            IF lace  IN viewPort.modes THEN INC(clickY,cy) ELSE INC(clickY,cy DIV 2) END;
            IF hires IN viewPort.modes THEN INC(clickX,cx) ELSE INC(clickX,cx DIV 2) END;
            IF clickY<0 THEN clickY := 0 END;
            IF clickX<0 THEN clickX := 0 END;
            IF clickY>=height THEN clickY := height-1 END;
            IF clickX>=width  THEN clickX := width -1 END;
            IF lace  IN viewPort.modes THEN mym := clickY+topEdge ELSE mym := 2*(clickY+topEdge) END;
            IF hires IN viewPort.modes THEN mxm := clickX ELSE mxm := 2*clickX END;
            Signal(Me,LONGSET{LMBSig});
          END;
          ev := nextEvent;
        END;
      END;
    END;
    WITH iB^ DO maxXMouse := mxm; maxYMouse := mym END;
  END;
  RETURN Ev;
END MyHandler; (* $S+ *)

(*------  VertB-Interrupt:  ------*)

PROCEDURE MyIntProc(); (* $S- *)

BEGIN
  INLINE(MOVEMS,3F3EH); (* this is MOVEM d2-d7/a2-a6,-(sp) *)

  IF (iB^.activeScreen = WBS) AND (WBS^.mouseY>=0) THEN
    WITH iB^ DO
      maxXMouse := mxm;
      maxYMouse := mym;
      IF aPointer#GetSpr THEN
        ActSprImg := 1-ActSprImg;
        GetSpr := aPointer;
        SprImg[ActSprImg]^ := GetSpr^;
        sprite.height := aPtrHeight;
        ChangeSprite(ADR(WBS^.viewPort),ADR(sprite),SprImg[ActSprImg]);
      END;
    END;
    IF (WBS^.mouseX#lx) OR (WBS^.mouseY#ly) THEN
      lx := WBS^.mouseX; ly := WBS^.mouseY;
      IF (ly>=0) AND (lx>=0) AND (ly<SizeY) AND (lx<SizeX) THEN
        WITH WBS^.viewPort.rasInfo^ DO
          IF Proportional THEN
            rxOffset := LONGINT(lx+1) * LONGINT(SizeX - oldw) DIV SizeX;
            ryOffset := LONGINT(ly+1) * LONGINT(SizeY - oldh) DIV SizeY;
            mx := lx - rxOffset;
            my := ly - ryOffset;
            ScrollVPort(ADR(WBS^.viewPort));
          ELSE
            IF (rox#rxOffset) OR (roy#ryOffset) OR (rox=-1)THEN
              IF rox=-1 THEN rox := 0 END;
              rxOffset := rox;
              ryOffset := roy;
              ScrollVPort(ADR(WBS^.viewPort));
            END;
            mx := lx - rox;
            my := ly - roy;
          END;
          IF hires IN WBS^.viewPort.modes THEN
            INC(mx,2*ORD(CAST(Byte,iB^.aXOffset)));
          ELSE
            INC(mx,ORD(CAST(Byte,iB^.aXOffset)));
          END;
          IF lace  IN WBS^.viewPort.modes THEN
            INC(my,2*ORD(CAST(Byte,iB^.aYOffset)));
          ELSE
            INC(my,ORD(CAST(Byte,iB^.aYOffset)));
          END;
          MoveSprite(ADR(WBS^.viewPort),ADR(sprite),mx,my);
        END;
      END;
    END;
  ELSE
    sprite.height := 1;
    ChangeSprite(ADR(WBS^.viewPort),ADR(sprite),EmptySprite);
    GetSpr := NIL;
  END;

  INLINE(MOVEML,7CFCH); (* this is MOVEM (sp)+,d2-d7/a2-a6 *)
END MyIntProc; (* $S+ *)

(*------  Neue MoveSprite() Funktion:  ------*)

(* $S- *)
PROCEDURE MyMoveSprite(vp{8}:ViewPortPtr; sprite{9}:SimpleSpritePtr; x{0},y{1}:INTEGER);
BEGIN
  INLINE(MOVEMS,3F3EH);
  IF (sprite^.num=0) AND (iB^.activeScreen=WBS) AND (WBS^.mouseY>=0) THEN
    x := -32;
  END;
  OldMoveSprite();
  INLINE(MOVEML,7CFCH);
END MyMoveSprite;
(* $S+ *)

(*------  CleanUp:  ------*)

PROCEDURE CleanUp();

BEGIN

(*------  Remove Inputhandler:  ------*)

  IF HandlerActive THEN
    WITH InputRequestBlock^ DO
      command := remHandler;
      data := ADR(HandlerStuff);
    END;
    DoIO(InputRequestBlock);
  END;
  IF InputRequestBlock#NIL THEN DeleteStdIO(InputRequestBlock) END;
  IF InputDevPort#NIL THEN DeletePort(InputDevPort) END;

(*------  Remove Interrupt:  ------*)

  IF IntActive THEN RemIntServer(vertb,ADR(VertBIntr)) END;

(*------  Remove Copperlist:  ------*)

  IF cl#NIL THEN
    FreeCopList(cl^.firstCopList);
    WBS^.viewPort.uCopIns:=altcl;
  END;

(*------  Reset Workbench:  ------*)

  IF WBS#NIL THEN
    WITH oldbm DO
      l := 0;
      WHILE l<LONGINT(depth) DO
        IF planes[l]=NIL THEN
          planes[l] := Exec.AllocMem(LONGINT(rows)*LONGINT(bytesPerRow),MemReqSet{chip,memClear});
          IF planes[l]=NIL THEN depth := l END;
        END;
        INC(l);
      END;
    END;
(* Korn, Bier, Schnaps und Wein *)
    Forbid();
      WITH WBS^ DO
        width := oldw;
        height := oldh;
        bitMap := oldbm;
        IF blitted THEN
          l := BltBitMap(ADR(bm),0,0,ADR(bitMap),0,0,oldw,oldh,0C0H,3,NIL);
        END;
        WITH viewPort.rasInfo^ DO rxOffset := 0; ryOffset := 0 END;
      END;
(* und wir hren unsere Leber Schrein. *)
      IF oldmy#0 THEN
        WITH iB^ DO maxXMouse := oldmx; maxYMouse := oldmy END;
      END;
      MakeScreen(WBS);
    Permit();
    RethinkDisplay();
  END;

(*------  Close everything:  ------*)

  IF OldMoveSprite#NIL THEN
    OldMoveSprite := SetFunction(ADR(Graphics),-426,CAST(ADDRESS,OldMoveSprite));
  END;
(* Doch eins das wissen wir ganz genau: *)
  IF Window#NIL THEN CloseWindow(Window) END;
  IF sprID#-1 THEN FreeSprite(sprID) END;
  IF LMBSig#-1 THEN FreeSignal(LMBSig) END;

(*------  Remove Port:  ------*)

  IF MyPort#NIL THEN
    Forbid();
      IF QuitMessage=NIL THEN QuitMessage := GetMsg(MyPort) END;
      WHILE QuitMessage#NIL DO
        IF QuitMessage^.msg.length=1 THEN QuitMessage^.cq := cqok END;
(* Ohne Alk da wre der Alltag so grau! *)
        ReplyMsg(QuitMessage);
        QuitMessage := GetMsg(MyPort);
      END;
      DeletePort(MyPort);
(* (ich war Montag auf dem Hosen Konzert!) *)
    Permit();
  END;

END CleanUp;

(*------  MAIN:  ------*)

BEGIN

(*------  Initialization:  ------*)

  WBS := NIL; Window := NIL; MyPort := NIL; blitted := FALSE;
  sprID := -1; InputDevPort := NIL; InputRequestBlock := NIL;
  HandlerActive := FALSE; InputOpen := FALSE; OldMoveSprite := NIL;
  SizeX := 1024; SizeY := 512; LMBSig := -1; oldmy := 0; cl := NIL;

  iB  := ADR(Intuition);
  gfx := ADR(Graphics);

  TermProcedure(CleanUp);

(*------  Have we already been started?  ------*)

  OldPort := FindPort(ADR(PortName));
  IF OldPort#NIL THEN
    MyPort := CreatePort(ADR(ReplyName),0);
    Assert(MyPort#NIL,ADR(oom));
    MyMsg.msg.node.type := message;
    MyMsg.msg.replyPort := MyPort;
    MyMsg.msg.length := 1;
    PutMsg(OldPort,ADR(MyMsg)); (* Signal task to quit *)
    WaitPort(MyPort);
    DeletePort(MyPort);
    MyPort := NIL;
    CASE MyMsg.cq OF
    cqoom: Assert(FALSE,ADR(oom)) |
    cqwin: Assert(FALSE,ADR(wins)) |
    ELSE Terminate(0) END;
  END;
  MyPort := CreatePort(ADR(PortName),0);
  Assert(MyPort#NIL,ADR(oom));

(*------  Get Arguments:  ------*)

  IF wbStarted THEN
    GetArg(0,arg,i);
    MyIcon := GetDiskObject(ADR(arg));
    IF MyIcon#NIL THEN
      tT := FindToolType(MyIcon^.toolTypes,ADR(tTWidth));
      IF tT#NIL THEN StrToVal(tT^,SizeX,b,10,err); Assert(NOT err,ADR(tTErr)) END;
      tT := FindToolType(MyIcon^.toolTypes,ADR(tTHeight));
      IF tT#NIL THEN StrToVal(tT^,SizeY,b,10,err); Assert(NOT err,ADR(tTErr)) END;
(* Ein Skinhead auf der Autobahn, Hui! Da kann man prima drberfahr! [Brieftauben] *)
      tT := FindToolType(MyIcon^.toolTypes,ADR(tTFlags));
      IF tT#NIL THEN
        Proportional :=  MatchToolValue(tT,ADR(tTProp))#NIL;
      END;
      FreeDiskObject(MyIcon);
    END;
  ELSE
    Assert(NumArgs()<4,ADR(Usage));
    Proportional := FALSE;
    IF ODD(NumArgs()) THEN
      GetArg(1,arg,i);
      Assert((CAP(arg[0])="P") AND (arg[1]=0C),ADR(Usage));
      Proportional := TRUE;
      l := 1;
    ELSE
      l := 0;
    END;
    IF NumArgs()>1 THEN
      GetArg(l+1,arg,i); StrToVal(arg,SizeX,b,10,err); Assert(NOT err,ADR(Usage));
      GetArg(l+2,arg,i); StrToVal(arg,SizeY,b,10,err); Assert(NOT err,ADR(Usage));
    END;
  END;

(*------  Open Window:  ------*)

  WITH NuWindow DO
    leftEdge   := 0; topEdge := 0;
    width      := 1; height  := 1;
    idcmpFlags := IDCMPFlagSet{};
    flags      := WindowFlagSet{backDrop};
    firstGadget:= NIL; checkMark := NIL;
    title      := ADR(WindowTitle);
    screen     := NIL; bitMap    := NIL;
    type       := ScreenFlagSet{wbenchScreen};
  END;
  Window := OpenWindow(NuWindow);
  Assert(Window#NIL,ADR(oom));

(*------  Allocate Sprite:  ------*)

  AllocMem(SprImg[0],SIZE(GetSpr^),TRUE);
  AllocMem(SprImg[1],SIZE(GetSpr^),TRUE);
  AllocMem(EmptySprite,SIZE(EmptySprite^),TRUE);
  Assert((SprImg[0]#NIL) AND (SprImg[1]#NIL),ADR(oom));
  ActSprImg := 0;
  sprID := GetSprite(ADR(sprite),-1);
  Assert(sprID>0,ADR(noSprites));
  OldMoveSprite := SetFunction(ADR(Graphics),-426,ADR(MyMoveSprite));

(*------  Signal:  ------*)

  LMBSig := AllocSignal(-1);
  Assert(LMBSig#-1,ADR(noSignal));
  Me := FindTask(NIL);
  IF SetTaskPri(Me,2)=0 THEN END; (* I'm responsible for quick WBDisplay! *)

(*------  Resize Workbench:  ------*)

  GetPrefs(ADR(pref),SIZE(Preferences));
  WITH Window^.wScreen^ DO
    oldbm:= bitMap;
    bm   := oldbm;
    oldw := width;
    oldh := height;
    Assert((SizeX>=oldw) AND (SizeY>=oldh),ADR(TooSmall));
  END;
  WITH bm DO
    rows := SizeY;
    bytesPerRow := ((SizeX+15) DIV 16) * 2;
    FOR l:=0 TO depth-1 DO
      AllocMem(planes[l],LONGINT(rows+10)*LONGINT(bytesPerRow),TRUE);
      Assert(planes[l]#NIL,ADR(oom));
    END;
  END;
  WBS := Window^.wScreen;
  Forbid();
  WaitBlit();
  WITH WBS^ DO
    l := BltBitMap(ADR(oldbm),0,0,ADR(bm),0,0,width,height,0C0H,3,NIL);
    blitted := TRUE;
    bitMap := bm;
    oldw := width; oldh := height; width := SizeX; height := SizeY;
    MakeScreen(WBS);
    WITH oldbm DO
      l := 0;
      WHILE l<LONGINT(depth) DO
        FreeMem(planes[l],LONGINT(rows)*LONGINT(bytesPerRow));
        planes[l] := NIL;
        INC(l);
      END;
    END;
    WITH iB^ DO
      oldmx := maxXMouse;
      oldmy := maxYMouse;
      Permit();
      IF l>1 THEN
        l := (sprID DIV 2) * 4 + 16;
        WITH pref DO
          SetRGB4(ADR(viewPort),l+1,
            CAST(INTEGER,CAST(BITSET,color17)*{8..11}) DIV 256,
            CAST(INTEGER,CAST(BITSET,color17)*{4.. 7}) DIV 16,
            CAST(INTEGER,CAST(BITSET,color17)*{0.. 3}));
          SetRGB4(ADR(viewPort),l+2,
            CAST(INTEGER,CAST(BITSET,color18)*{8..11}) DIV 256,
            CAST(INTEGER,CAST(BITSET,color18)*{4.. 7}) DIV 16,
            CAST(INTEGER,CAST(BITSET,color18)*{0.. 3}));
          SetRGB4(ADR(viewPort),l+3,
            CAST(INTEGER,CAST(BITSET,color19)*{8..11}) DIV 256,
            CAST(INTEGER,CAST(BITSET,color19)*{4.. 7}) DIV 16,
            CAST(INTEGER,CAST(BITSET,color19)*{0.. 3}));
        END;
      END;
      sprite.x := 0;
      sprite.y := 0;
      sprite.height := aPtrHeight;
      GetSpr := aPointer;
      SprImg[ActSprImg]^ := GetSpr^;
    END;
    ChangeSprite(ADR(viewPort),ADR(sprite),SprImg[ActSprImg]);
  END;

(*------  Create Copperlist:  ------*)

  altcl:=WBS^.viewPort.uCopIns;
  AllocMem(cl,SIZE(cl^),TRUE);
  Assert(cl#NIL,ADR(oom));
  CMove(cl,ADR(custom.dmacon),CAST(INTEGER,8100H)); CBump(cl); (* Bitplane-DMA ein *)
  CWait(cl,oldh,0); CBump(cl);
  CMove(cl,ADR(custom.dmacon),0100H); CBump(cl); (* Bitplane-DMA aus *)
  CWait(cl, 10000, 255); CBump(cl);
  WBS^.viewPort.uCopIns:=cl;
  RethinkDisplay();

(*------  Add Inputhandler:  ------*)

  InputDevPort := CreatePort(NIL,0);
  Assert(InputDevPort#NIL,ADR(oom));

  InputRequestBlock := CreateStdIO(InputDevPort);
  Assert(InputRequestBlock#NIL,ADR(oom));

  OpenDevice(ADR(inputName),0,InputRequestBlock,LONGSET{});
  IF InputRequestBlock^.error#0 THEN Terminate(0) END;
  InputOpen := TRUE;

  WITH HandlerStuff DO
    data := NIL;
    code := ADR(MyHandler);
    node.pri := 51;
  END;
  WITH InputRequestBlock^ DO
    command := addHandler;
    data := ADR(HandlerStuff);
  END;
  DoIO(InputRequestBlock);
  HandlerActive := TRUE;

(*------  Interrupt starten:  ------*)

  WITH VertBIntr DO
    node.type := interrupt;
    node.pri  := 0;
    node.name := ADR(intName);
    data := NIL;
    code := MyIntProc;
  END;
  AddIntServer(vertb,ADR(VertBIntr));
  IntActive := TRUE;

(*------  Do it:  ------*)

  lx := -1; ly := -1; rox := -1; roy := 0;
  WITH WBS^ DO
    mxm := width-1; mym := height-1;
    IF NOT(lace  IN viewPort.modes) THEN INC(mym,mym) END;
    IF NOT(hires IN viewPort.modes) THEN INC(mxm,mxm) END;
  END;
  LOOP
    REPEAT
      Sigs := Wait(LONGSET{MyPort^.sigBit,LMBSig});
      IF LMBSig IN Sigs THEN
        lay:= WhichLayer(ADR(WBS^.layerInfo),clickX,clickY);
        IF (lay#NIL) THEN
          IF lay^.window=NIL THEN  (* Screentitlebar *)
            newmxm := WBS^.width - 1;
            IF clickY<12 THEN newmym := oldh - 1 ELSE newmym := WBS^.height - 1 END;
          ELSE
            lw := lay^.window;
            WITH lw^ DO
              gdg:= firstGadget; DEC(clickX,leftEdge); DEC(clickY,topEdge);
            END;
            LOOP
              IF gdg=NIL THEN
                WITH WBS^ DO
                  newmxm := width  - 1;
                  newmym := height - 1;
                  EXIT;
                END;
              END;
              WITH gdg^ DO
                gy := topEdge;  IF gRelBottom IN flags THEN INC(gy,lw^.height) END;
                gx := leftEdge; IF gRelRight  IN flags THEN INC(gx,lw^.width ) END;
                gh := height;   IF gRelHeight IN flags THEN INC(gh,lw^.height) END;
                gw := width;    IF gRelWidth  IN flags THEN INC(gw,lw^.width ) END;
                IF (gx<=clickX) AND (gy<=clickY) AND (gx+gw>clickX) AND (gy+gh>clickY) THEN
                  CASE CAST(INTEGER,CAST(BITSET,gadgetType)*{4..7}) OF
                  sizing:
                    WITH lw^ DO
                      IF maxWidth#-1 THEN
                        newmxm := leftEdge + maxWidth + clickX - width;
                        IF newmxm>=WBS^.width THEN newmxm := WBS^.width - width + clickX END;
                      ELSE
                        newmxm := WBS^.width - width + clickX;
                      END;
                      IF maxHeight#-1 THEN
                        newmym := topEdge + maxHeight + clickY - height;
                        IF newmym>=WBS^.height THEN newmym := WBS^.height - height + clickY END;
                      ELSE
                        newmym := WBS^.height - height + clickY;
                      END;
                    END;
                    INC(newmym,WBS^.topEdge);
                    EXIT |
                  wDragging:
                    WITH WBS^ DO
                      newmxm := width  - lw^.width  + clickX;
                      newmym := height - lw^.height + clickY + topEdge;
                    END;
                    EXIT |
                  ELSE END;
                END;
                gdg := nextGadget;
              END;
            END;
          END;
          WITH WBS^.viewPort DO
            IF lace  IN modes THEN mym := newmym ELSE mym := 2*newmym END;
            IF hires IN modes THEN mxm := newmxm ELSE mxm := 2*newmxm END;
          END;
        END;
      END;
      QuitMessage := GetMsg(MyPort);
    UNTIL QuitMessage#NIL;

    Forbid();
    w := WBS^.firstWindow;
    WHILE LONGCARD(w)>1 DO
      WITH w^ DO
        IF (width+leftEdge>oldw) OR (height+topEdge>oldh) THEN
          w := WindowPtr(1);
        ELSE
          w := nextWindow;
        END;
      END;
    END;
    Permit();

    IF w=NIL THEN
      WITH oldbm DO
        IF planes[0]=NIL THEN
          planes[0] := Exec.AllocMem(LONGINT(rows)*LONGINT(bytesPerRow),MemReqSet{chip,memClear});
          IF planes[0]#NIL THEN EXIT ELSIF QuitMessage^.msg.length=1 THEN QuitMessage^.cq := cqoom END;
        END;
      END;
    ELSIF QuitMessage^.msg.length=1 THEN QuitMessage^.cq := cqwin END;

    ReplyMsg(QuitMessage);
    QuitMessage := NIL;
  END;

END MegaWB.
