(*---------------------------------------------------------------------------
   :Program.    PopUp-Module
   :Version.    2.01 as Modula-2 modules pop.def and pop.mod.
   :Contants.   Usage of Amiga PopUp menus.
   :Remark.     Works similar to Intuition menus.
   :History.    V1.0, pop.c by Derek Zahn,
   :History.                   (Gambit Software, Madison WI) July 1987
   :History.    V2.0,  1:1  conversion to Modula-2 and
   :History.    V2.01, rectified and slightly improved version by
   :Author.     Jochen P. Kupfer,
   :Address.    Buchenweg 22, D-4006 Erkrath 2, West-Germany,
   :Phone.      2104-40673
   :Group.      SIGMA (Special Interest Group Modula-2 Amiga)
   :Date.       4/16-Feb-89
   :Copyright.  PD
   :Language.   Modula-2
   :Translator. M2Amiga V3.2d
---------------------------------------------------------------------------*)


IMPLEMENTATION MODULE pop;

FROM Intuition IMPORT NewWindow, OpenWindow, CloseWindow, WindowPtr,
                      MenuPtr, IDCMPFlagSet, IntuiMessagePtr, ScreenPtr,
                      IntuitionBase, MenuItemFlags, MenuItemPtr,
                      DrawImage, PrintIText, IDCMPFlags, WindowFlags,
                      selectDown, selectUp, menuDown, menuUp,
                      WindowFlagSet, customScreen, MenuItemFlagSet;
FROM Graphics  IMPORT SetDrMd, DrawModes, DrawModeSet, Draw, Move,
                      SetAPen, SetBPen, RectFill, jam1, RastPortPtr,
                      Text;
FROM GfxMacros IMPORT SetAfPen;
FROM Exec      IMPORT GetMsg, ReplyMsg, Wait, MsgPortPtr;
FROM SYSTEM    IMPORT LONGSET, ADDRESS, ADR;
FROM Arts      IMPORT TermProcedure, CurrentLevel;
                                  (* ^-Version 3.2 only *)
TYPE
  str   = ARRAY[0..79] OF CHAR;
  strPtr= POINTER TO str;

VAR
  popGhostPattern,
  popNormalPattern :ARRAY[0..1] OF CARDINAL;
  popWindow        :NewWindow;
  popwinPtr        :WindowPtr;
  thisLevel        :INTEGER;  (* Only needed for debugging (V.3.2) *)

(*
 * Length()
 *
 * a home-brewed Lenght() to prevent it being necessary to hook in whatever
 * huge object file in which the M2 library's Length() resides.
 * Length() from Strings couldn't be used anyway 'cause we have only the
 * string's ADDRESS!
 *)


PROCEDURE Length(t:strPtr):INTEGER;
  VAR
    i :INTEGER;
  CONST
    nul = 00C;
  BEGIN
    i := 0;
    WHILE (t^[i] # nul) DO   (* the string must be nul-terminated! *)
      i := i+1;
    END;
    RETURN i;
  END Length;


(*
 * PopDrawMenuItem()
 *
 * Draws the specified menuItem in the given rastPort.  The mode argument
 * says what to draw -- FALSE means draw the ItemFill, TRUE the SelectFill.
 *)

PROCEDURE PopDrawMenuItem(winPtr:WindowPtr;itemPtr:MenuItemPtr;
                          mode:BOOLEAN;offset:INTEGER);
  VAR
    fill        :ADDRESS;
    rPtr        :RastPortPtr;
  BEGIN
   (* first, figure out what to do, and RETURN if it is a NIL thing *)

   fill := NIL;
   IF NOT mode THEN fill := itemPtr^.itemFill
   ELSE   fill := itemPtr^.selectFill
   END;
   IF fill=NIL THEN RETURN END;

   rPtr := winPtr^.rPort;

        (* First, erase what may already be there, just to be sure that    *)
        (* everything works out all right.                                 *)
   SetAPen(rPtr,1);
   SetDrMd(rPtr,jam1);
   RectFill(rPtr,itemPtr^.leftEdge,
                 itemPtr^.topEdge + offset,
                 itemPtr^.leftEdge + itemPtr^.width,
                 itemPtr^.topEdge + itemPtr^.height + offset);

        (* Now, draw the item itself -- depending on the Flag value, it    *)
        (* could be either an Image or an IntuiText                        *)

   IF (itemText IN itemPtr^.flags) THEN
      PrintIText(rPtr,fill,itemPtr^.leftEdge,itemPtr^.topEdge + offset);
   ELSE
      DrawImage(rPtr,fill,itemPtr^.leftEdge,itemPtr^.topEdge + offset);

        (* If the itemEnabled flag is not set, "ghost" the item.           *)
   END;

        (* If the item is checkmarked, draw the checkMark.  Intuition made *)
        (* sure that the checkMark field of the window RECORD exists.      *)
        (* Changed to make sure checkMark is drawn after(!) the item.      *)

   IF (checkIt IN itemPtr^.flags) THEN
      IF (checked IN itemPtr^.flags) THEN
         DrawImage(rPtr,winPtr^.checkMark,itemPtr^.leftEdge+1,
                        itemPtr^.topEdge + offset +1);
      END;
   END;

   IF NOT(itemEnabled IN itemPtr^.flags) THEN
      SetAPen(rPtr,1);
      SetDrMd(rPtr,jam1);
      SetAfPen(rPtr,ADR(popGhostPattern),1);
      RectFill(rPtr,itemPtr^.leftEdge,
                    itemPtr^.topEdge + offset,
                    itemPtr^.leftEdge + itemPtr^.width,
                    itemPtr^.topEdge + itemPtr^.height + offset);
      SetAfPen(rPtr,ADR(popNormalPattern),1);
   END;

   RETURN;

 END PopDrawMenuItem;


(*
 * PopRender()
 *
 * renders the menu title (if existent) and the menu items.
 *)

PROCEDURE PopRender(winPtr:WindowPtr;menPtr:popMenuPtr);
  VAR
    itemPtr    :MenuItemPtr;
    rPtr       :RastPortPtr;
    offset     :INTEGER;
  CONST
    compl = DrawModeSet{complement};
  BEGIN
    offset := 0;
    rPtr := winPtr^.rPort;

    (* Fill the background with color 1, like Intuition Menus *)

    SetAPen(rPtr,1);
    RectFill(rPtr,0,0,winPtr^.width,winPtr^.height);

    (* First, if there is a Title for this menu, render it in the top *)
    (* of the menu.                                                   *)

    IF menPtr^.menuName#NIL THEN
       SetDrMd(rPtr,jam1);
       SetAPen(rPtr,0);
       SetBPen(rPtr,1);
       Move(rPtr,4,7);
       Text(rPtr,menPtr^.menuName,Length(menPtr^.menuName));
       SetDrMd(rPtr,compl);
       RectFill(rPtr,0,0,winPtr^.width,PopTitleHeight);
       SetDrMd(rPtr,jam1);
       offset := PopTitleHeight;
    END;

    (* now render all of the menu items *)

    itemPtr := menPtr^.firstItem;
    WHILE itemPtr#NIL DO
      PopDrawMenuItem(winPtr,itemPtr,FALSE,offset);
      itemPtr := itemPtr^.nextItem;
    END;
  END PopRender;

(*
 * PopGetIitem()
 *
 * given the traversal number of a menu item in a menu (assumes, BTW, that
 * the arguments are valid), return a pointer to the MenuItem structure
 *)

PROCEDURE PopGetItem(menPtr:popMenuPtr;which:INTEGER):MenuItemPtr;
  VAR
    itemPtr  :MenuItemPtr;
  BEGIN
    itemPtr := menPtr^.firstItem;
    WHILE which#0 DO
      DEC(which);
      itemPtr := itemPtr^.nextItem;
    END;
    RETURN itemPtr;
  END PopGetItem;

(*
 * PopComputeState()
 *
 * This function checks to see where the mouse pointer is in relation to
 * the various menu items in the menu.  If it is inside one of them, it
 * returns which one (indexed by its linear position in the MenuItem list
 * with 0 being the first one).  If not, returns -1.
 *
 * Possible future enhancement: keep a set of state variables containing
 * the UL and LR corners of the last-known select box; this would make
 * a quick check possible and would cut down the computation for short
 * mouse movements (the most common).
 *)


PROCEDURE PopComputeState(winPtr:WindowPtr;menPtr:popMenuPtr):INTEGER;
  VAR
    current,
    xval,yval :INTEGER;
    itemPtr   :MenuItemPtr;
  BEGIN
    current := 0;
               (* Get the x and y vals of the mouse position *)
    xval := winPtr^.mouseX;
    yval := winPtr^.mouseY;

       (* If there is a title, decrement the yval by the correct amount *)
    IF menPtr^.menuName#NIL THEN yval := yval - PopTitleHeight END;

       (* First, see if the pointer is even in the window *)

    IF ((xval<0) OR (yval<0) OR
        (xval>winPtr^.width) OR (yval>winPtr^.height))
        THEN RETURN -1
    END;

    (* search through the list of menu items, checking the select box  *)
    (* of each.  If containment is detected, the job is done.          *)

    itemPtr := menPtr^.firstItem;
    WHILE (itemPtr#NIL) DO
      IF ((xval >= itemPtr^.leftEdge) AND (yval >= itemPtr^.topEdge)
         AND (xval <= itemPtr^.leftEdge + itemPtr^.width)
         AND (yval <= itemPtr^.topEdge  + itemPtr^.height)) THEN

                        (* We have found the quarry; now, the result only  *)
                        (* depends on the MenuItem's itemEnabled flag.     *)

              IF (itemEnabled IN itemPtr^.flags) THEN
                RETURN current
              ELSE
                RETURN -1
              END;
      END; (* IF *)
      INC(current);
      itemPtr := itemPtr^.nextItem;
    END; (* WHILE *)

    (* If the list is exhausted, return the sad news *)

    RETURN -1;
  END PopComputeState;

(*
 * PopDoHighLighting()
 *
 * HighLight or UnHighLight a menu item, given its traversal number.  Assumes
 * this is a rational value -- if it isn't, Watch Out.
 *)

PROCEDURE PopDoHighLighting(winPtr:WindowPtr;menPtr:popMenuPtr;
                            state:INTEGER;
                            mode:BOOLEAN);
              (* mode: FALSE means to highlight,
                       TRUE  means to unhighlight *)
  VAR
    itemPtr     :MenuItemPtr;
    rPtr        :RastPortPtr;
    offset      :INTEGER;
  CONST
    compl = DrawModeSet{complement};
  BEGIN
    offset := 0;
    IF menPtr^.menuName#NIL THEN offset := PopTitleHeight END;

        (* Get the correct MenuItem RECORD *)

    itemPtr := PopGetItem(menPtr,state);
    rPtr    := winPtr^.rPort;

        (* Now, do the HighLighting!  The action to be taken depends on    *)
        (* the type of HighLighting desired for this item.                 *)
        (* The way that the flags for HighLighting works is truly bizarre  *)
        (* Now it's even ***more*** bizarre!                J.             *)

    IF ((highBox IN itemPtr^.flags) AND
        (highComp IN itemPtr^.flags)) THEN RETURN END;
                                         (* = highNone *)

     (*  With an additional IF, HighLighting of
         itemPtr^.itemFill is now possible!       J. (4-2-89)         *)

    IF NOT mode THEN
      PopDrawMenuItem(winPtr,itemPtr,TRUE,offset);
      IF highComp IN itemPtr^.flags THEN
         SetDrMd(rPtr,compl);
         RectFill(rPtr,itemPtr^.leftEdge,
                       itemPtr^.topEdge + offset,
                       itemPtr^.leftEdge + itemPtr^.width - 1,
                       itemPtr^.topEdge + itemPtr^.height + offset);
      ELSIF highBox IN itemPtr^.flags THEN
        SetDrMd(rPtr,compl);
        Move(rPtr,itemPtr^.leftEdge,itemPtr^.topEdge + offset);
        Draw(rPtr,itemPtr^.leftEdge + itemPtr^.width -1,
                  itemPtr^.topEdge + offset);
        Draw(rPtr,itemPtr^.leftEdge + itemPtr^.width -1,
                  itemPtr^.topEdge + itemPtr^.height + offset);
        Draw(rPtr,itemPtr^.leftEdge,
                itemPtr^.topEdge + itemPtr^.height + offset);
        Draw(rPtr,itemPtr^.leftEdge,itemPtr^.topEdge + offset);
      END;
    ELSE
      PopDrawMenuItem(winPtr,itemPtr,NOT mode,offset);
    END; (* IF *)

    RETURN;

  END PopDoHighLighting;

(*
 * PopHighLight()
 *
 * HighLight a menuItem
 *)

PROCEDURE PopHighLight(winPtr:WindowPtr;menPtr:popMenuPtr;state:INTEGER);
  BEGIN
    PopDoHighLighting(winPtr,menPtr,state,FALSE);
  END PopHighLight;

(*
 * PopUnHighLight()
 *
 * UnHighLight a menuItem
 *)

PROCEDURE PopUnHighLight(winPtr:WindowPtr;menPtr:popMenuPtr;state:INTEGER);
  BEGIN
    PopDoHighLighting(winPtr,menPtr,state,TRUE);
  END PopUnHighLight;

(*
 * PopChoose(menPtr, winPtr)
 * menPtr -- pointer to the menu to pop
 * winPtr -- the window to which this menu relates.
 *           NIL is not allowed, since I don't see any use for a
 *           PopUpWindow for a window which is not open!
 *
 * This function provides a blocking pop-up menu.  It returns  -1 if
 * either an error occurred attempting to pop or if no selection was made
 * by the user.  If a selection was made, a number between 0 and n-1, where
 * n is the number of Menu Items.
 *
 * -1 is also returned if a selection of a checked item was made.
 *
 * Since this code opens a window, it is up to the caller to be sure that
 * no scribbling in droll ways is done while this code is in progress.
 *)


PROCEDURE PopChoose(menPtr:popMenuPtr;winPtr:WindowPtr):LONGINT;
  VAR
    sig,exclude  :LONGSET;         (* for handling mutual exclusions     *)
    im           :IntuiMessagePtr; (* our eyes and ears                  *)
    class        :IDCMPFlagSet;    (* incoming IntuiMessage class        *)
    code,                          (* incoming IntuiMessage code         *)
    popNewState,
    popState,i   :INTEGER;         (* menu selection state variables     *)
    finished,                      (* set when menu should be blown away *)
    mouseMoved   :BOOLEAN;         (* keeps track of mouse movements     *)
    selItem      :MenuItemPtr;     (* the selected Item                  *)
    popup        :MsgPortPtr;      (* Ptr to our userPort                *)
    popwinPtr    :WindowPtr;       (* window of pop-up menu              *)
    screenPtr    :ScreenPtr;       (* screen of window win               *)
  BEGIN
        (* Check to see that our window and menu exist.                  *)
        (* While this is not any sort of guarantee against disaster, it  *)
        (* is better than nothing.                                       *)

    IF (winPtr=NIL) OR (menPtr=NIL) OR
        NOT(menuEnabled IN menPtr^.flags) THEN RETURN -1 END;

        (* Form the menu window to blast forth into the Visual World. Note *)
        (* the unconventional (and inconsistent with Intuition) ways that  *)
        (* the Width and Height fields are used here.                      *)

    popWindow.width  := menPtr^.width;   (* !!non-standard!!! *)
    popWindow.height := menPtr^.height;  (* !!non-standard!!! *)

                      (* Inherit CheckMark from the "parent" window *)
    IF winPtr^.checkMark#NIL THEN
       popWindow.checkMark := winPtr^.checkMark END;

    screenPtr := winPtr^.wScreen;
    popWindow.screen := screenPtr;
    popWindow.leftEdge := menPtr^.leftEdge;
    popWindow.topEdge  := menPtr^.topEdge;

        (* if we are supposed to return to the last-selected menuItem and  *)
        (* such a beast exists, all other positioning information (except  *)
        (* popTidy) will be circumvented.  The menu will appear under the  *)
        (* pointer with the last-chosen item pre-selected, if this is      *)
        (* possible given the popTidy flag and the screen constraints.     *)
        (* In this case, the LeftEdge and TopEdge fields of the menu       *)
        (* structure will have been altered (I know, ick!) to provide a    *)
        (* relative offset with respect to the pointer to do the deed      *)

    IF (popRemember IN menPtr^.flags) AND (popUsed IN menPtr^.flags) THEN
       popWindow.leftEdge := popWindow.leftEdge+screenPtr^.mouseX;
       popWindow.topEdge  := popWindow.topEdge+screenPtr^.mouseY;
    ELSE
       IF (popPointRel IN menPtr^.flags) THEN
          popWindow.leftEdge := popWindow.leftEdge+screenPtr^.mouseX;
          popWindow.topEdge  := popWindow.topEdge+screenPtr^.mouseY;
       ELSIF (popWinRel IN menPtr^.flags) THEN
          popWindow.leftEdge := popWindow.leftEdge+winPtr^.mouseX;
          popWindow.topEdge  := popWindow.topEdge+winPtr^.mouseY;
       END;
    END;

        (* If the caller wishes us to be popTidy, the menu must completely *)
        (* appear on the screen, whatever other effects this may have on   *)
        (* menu positioning.  The left edge and top edge must be altered   *)
        (* accordingly.  In the pathological case where the menu is larger *)
        (* than the screen, -1 is returned.                                *)
        (* If poptidiness is not a factor, the size of the window may have *)
        (* to be altered if it shoots off the bottom or right edge of the  *)
        (* screen.  There should be some similar mechanism to deal with    *)
        (* the menu if it extends past the top or left edge of the screen; *)
        (* as it stands now, the OpenWindow() call will fail, and the      *)
        (* result may be even more dire under 1.1.  Use 1.2!               *)

    IF (popTidy IN menPtr^.flags) THEN
       IF ((popWindow.width > screenPtr^.width) OR
           (popWindow.height > screenPtr^.height)) THEN RETURN -1
       END;
       IF (popWindow.leftEdge + popWindow.width > screenPtr^.width)
          THEN popWindow.leftEdge := screenPtr^.width - popWindow.width
       END;
       IF (popWindow.topEdge + popWindow.height > screenPtr^.height)
          THEN popWindow.topEdge := screenPtr^.height - popWindow.height
       END;
       IF popWindow.leftEdge < screenPtr^.leftEdge THEN
          popWindow.leftEdge := screenPtr^.leftEdge
       END;
       IF popWindow.topEdge < screenPtr^.topEdge THEN
          popWindow.topEdge := screenPtr^.topEdge
       END;

    ELSE
       IF (popWindow.leftEdge + popWindow.width > screenPtr^.width)
          THEN popWindow.width := screenPtr^.width - popWindow.leftEdge
       END;
       IF (popWindow.topEdge + popWindow.height > screenPtr^.height)
          THEN popWindow.height := screenPtr^.height - popWindow.topEdge
       END;
    END;

        (* There!  Finally, the window is ready to be displayed!  First,   *)
        (* create it.                                                      *)

     popwinPtr := OpenWindow(popWindow);
     IF popwinPtr = NIL THEN RETURN -1 END; (* all that work for nuthin'   *)

        (* Now, render the menu items and (possibly) the menu title.       *)

     PopRender(popwinPtr,menPtr);

        (* Now, see if the pointer is over a selection.  The variable      *)
        (* 'popState' will from this point on hold the value, in linear    *)
        (* traversal order of the MenuItems (zero-indexed), the currently  *)
        (* selected menuItem, or -1 if none are selected.                  *)

     popState := PopComputeState(popwinPtr,menPtr);

        (* If one is indeed currently selected, highlight it.              *)

     IF popState >= 0 THEN PopHighLight(popwinPtr,menPtr,popState) END;

        (* Here is the IDCMP loop that will process the pop-up menu.  Note *)
        (* that on mousemove events, I don't care where it moved, just if  *)
        (* it did -- PopComputestate() will figure out where by reaching   *)
        (* into the Window RECORD.  Not Pure Programming, somehow, but     *)
        (* blessed by the Intuition manual.                                *)

     finished := FALSE;
     popup := popwinPtr^.userPort;
     LOOP    (* while(1) ... *)
       mouseMoved := FALSE;
       sig := Wait(LONGSET{popup^.sigBit});
       im  := GetMsg(popup);
       class := im^.class;
       code  := im^.code;
       ReplyMsg(im);
                        (* The only messages we should be getting are      *)
                        (* mouseButton and move events.  Button events     *)
                        (* could signify the end of this routine's         *)
                        (* epheremal spotlight role.                       *)
       IF (mouseMove IN class) THEN mouseMoved := TRUE END;
       IF (mouseButtons IN class) THEN
          CASE code OF
            | selectDown : IF ((popLeftButton IN menPtr^.flags) AND
                               (popTriggerDown IN menPtr^.flags))
                           THEN finished := TRUE END;
            | selectUp   : IF ((popLeftButton IN menPtr^.flags) AND
                               (popTriggerUp IN menPtr^.flags))
                           THEN finished := TRUE END;
            | menuDown   : IF ((popRightButton IN menPtr^.flags) AND
                               (popTriggerDown IN menPtr^.flags))
                           THEN finished := TRUE END;
            | menuUp     : IF ((popRightButton IN menPtr^.flags) AND
                               (popTriggerUp IN menPtr^.flags))
                           THEN finished := TRUE END;
            ELSE ; (* What else ??????? *)
         END;
       END; (* IF *)

                (* if the exit conditions have been met, we can return our *)
                (* results with honor and dignity, having served.          *)
                (* Note that if we are remembering the last selection, the *)
                (* menu structure is mangled to make that possible.        *)

       IF finished THEN
          popState := PopComputeState(popwinPtr,menPtr);
          IF popState >= 0 THEN
             IF popRemember IN menPtr^.flags THEN
                INCL(menPtr^.flags,popUsed);
                menPtr^.leftEdge := -1*popwinPtr^.mouseX;
                menPtr^.topEdge := -1*popwinPtr^.mouseY;
             END;

                        (* Special things to do if the menu entry  *)
                        (* is of type checkIt                      *)

             selItem := PopGetItem(menPtr,popState);
             IF checkIt IN selItem^.flags THEN
                IF checked IN selItem^.flags THEN
                   popState := -1;
                   IF menuToggle IN selItem^.flags THEN
                      EXCL(selItem^.flags,checked);

                      (* EXCL() doesn't work in V.3.11, there it has to be
                         circumvented by something like
                         selItem^.flags:=selItem^.flags
                                         -MenuItemFlagSet{checked};... J.*)

                   END;
                ELSE
                   INCL(selItem^.flags,checked);
                                    (* Handle mutual exclusion *)
                   exclude := selItem^.mutualExclude;
                   IF exclude#LONGSET{} THEN
                      selItem := menPtr^.firstItem;
                      i := 0;
                      WHILE selItem#NIL DO  (* changed to Modula usage *)
                        IF i IN exclude THEN
                           EXCL(selItem^.flags,checked); (* see above *)
                        END;
                        i := i + 1;
                        selItem := selItem^.nextItem;
                      END; (* WHILE *)
                   END; (* IF *)
                END; (* IF-ELSE *)
             END; (* IF *)
          END; (* IF *)
          CloseWindow(popwinPtr);
          RETURN popState;
       END; (* finished *)

                (* if the mouse has moved, find out its new state and      *)
                (* alter the HighLighting accordingly.                     *)

       IF mouseMoved THEN
          popNewState := PopComputeState(popwinPtr,menPtr);
          IF popNewState#popState THEN
             IF popState>=0 THEN
                PopUnHighLight(popwinPtr,menPtr,popState)
             END;
             IF popNewState>=0 THEN
                PopHighLight(popwinPtr,menPtr,popNewState)
             END;
             popState := popNewState;
          END;
        END; (* mouseMoved *)
      END; (* LOOP *)
   END PopChoose;


PROCEDURE CleanIt;
  BEGIN
    IF thisLevel >= CurrentLevel() THEN   (* for debugging only *)
      IF popwinPtr#NIL THEN CloseWindow(popwinPtr) END;
    END;
  END CleanIt;


BEGIN                    (* local MAIN *)
  thisLevel := CurrentLevel();  (* for debugging only *)
  TermProcedure(CleanIt);       (* M2Amiga special    *)

      (* Area fill patterns *)
  popGhostPattern[0]  := 01111H;popGhostPattern[1]  := 04444H;
  popNormalPattern[0] := 0FFFFH;popNormalPattern[1] := 0FFFFH;


(* This RECORD will be used to create a window for display of the          *)
(* menu.  In my heart of hearts, I wanted to use graphics library          *)
(* functions instead, but reason prevailed.  Note the use of the rmbTrap   *)
(* flag -- while the pop-up menu is being processed, there is no use for   *)
(* the right button.  Perhaps this should only be set if the right button  *)
(* has some bearing on the pop-up menu.                                    *)


  WITH popWindow DO
      leftEdge    := 0;  (* \                         *)
      topEdge     := 0;  (*  \                        *)
      width       := 0;  (*   \                       *)
      height      := 0;  (*    To be filled in later. *)
      detailPen   := -1; (* in V.3.11 use 0 *)
      blockPen    := -1; (* in V.3.11 use 1 *)
      idcmpFlags  := IDCMPFlagSet{mouseButtons,mouseMove};
      flags       := WindowFlagSet{windowRefresh,reportMouse,
                                   activate,rmbTrap};
      firstGadget := NIL;
      checkMark   := NIL; (* Maybe later *)
      title       := NIL;
      screen      := NIL; (*    To be filled in later. *)
      bitMap      := NIL; (* No custom BitMap *)
      minWidth    := 0;   (* No changes in size necessary. *)
      minHeight   := 0;
      maxWidth    := 0;
      maxHeight   := 0;
      type        := customScreen; (* always use this value *)
    END;


END pop.mod

