(*---------------------------------------------------------------------------
    :Program.     Area.mod
    :Contents.	  Simple und korrekte Benutzung von Graphics.Area..
    :Author.      Bernd Preusing
    :Address.	  Gerhardstr. 16  D-2200 Elmshorn
    :Phone.	  04121/22486
    :Copyright.	  Public Domain
    :Language.	  Modula-2
    :Translator.  M2Amiga V3.2e
    :History.	  V1.0 Preusing 27-Apr-89
    :Bugs.	  falls RastPort nicht initialisiert: Guru oder schlimmer
    :Remark.      Graphics.AreaMove etc. liefern ein Resultat zurck,
    :Remark.      dies wird hier mittels Arts.Assert unterdrckt.
    :Remark.      Gemischter Betrieb von Polygonen und Ellipsen fhrt
    :Remark.      unter graphics.library zu Fehlern und Abstrzen,
    :Remark.      dies Modul verhindert das.
---------------------------------------------------------------------------*)
IMPLEMENTATION MODULE Area;

FROM SYSTEM	IMPORT	ADR, CAST, ADDRESS;
FROM Arts	IMPORT	Assert;
IMPORT Graphics; (* wg Area.. (doppelte Bezeichner!) *)
FROM Graphics	IMPORT	RastPortPtr, AreaInfo, TmpRas, InitTmpRas;


(* $Z-  (Parameter gleich in die Register) *)

PROCEDURE InitArea(Rp: RastPortPtr; MaxVectors: INTEGER; Alloc: AllocProc);
(*	:Input.   Rp: zu initialisierender RastPort
	:Input.   MaxVectors: maximale Punktzahl vor AreaEnd. Falls
	:Input.		      mehr bentigt werden: Assert.
	:Input.		      Eine Ellipse bentigt nur 2 Punkte, 20
	:Input.		      bis 50 reicht also allemal!
	:Input.   Alloc: Prozedur zur Speicheranforderung (z.B. Heap.AllocMem)
	:Semantic. Der RastPort wird fr Area-Operationen initialisiert,
	:Semantic. alle ntigen Puffer werden alloziert. Die Verantwortung
	:Semantic. fr die Speicherfreigabe mu eine TermProcedure tragen!
*)
TYPE
  AreaStructPtr = POINTER TO RECORD (* etwas Aufwand sparen *)
				tr: TmpRas;
				ai: AreaInfo
			     END;
VAR
  TempPlane: ADDRESS;
  TrAndAi: AreaStructPtr;
  VectBuff: ADDRESS;
  RasSize: LONGINT;
BEGIN
  WITH Rp^.bitMap^ DO
    RasSize:=bytesPerRow*rows;
  END;

  Alloc(TempPlane,RasSize,TRUE);
  Assert(TempPlane#NIL,ADR('no mem: Area.tempPlane'));

  Alloc(TrAndAi,SIZE(TrAndAi^),FALSE);
  Assert(TrAndAi#NIL,ADR('no mem: Area.Structs'));

  Alloc(VectBuff,MaxVectors*5,FALSE);
  Assert(VectBuff#NIL,ADR('no mem: Area.VectBuff'));

  WITH TrAndAi^ DO
    Graphics.InitArea(ai,VectBuff,MaxVectors);
    InitTmpRas(tr,TempPlane,RasSize);
    WITH Rp^ DO
      tmpRas:=ADR(tr);
      areaInfo:=ADR(ai);
    END
  END;
END InitArea;


PROCEDURE AreaMove(Rp: RastPortPtr; x,y: INTEGER);
(*	:Semantic. Bewegt den Area-Zeiger nach x,y und
	:Semantic. schliet ein evtl. vorher begonnenes Polygon.
*)
BEGIN
  Assert(Graphics.AreaMove(Rp,x,y)=0,ADR('Err: AreaMove'));
END AreaMove;


PROCEDURE AreaDraw(Rp: RastPortPtr; x,y: INTEGER);
(*	:Semantic. Zeichnet eine Area-Linie nach x,y.
*)
BEGIN
  Assert(Graphics.AreaDraw(Rp,x,y)=0,ADR('Err: AreaDraw'));
END AreaDraw;


PROCEDURE AreaEllipse(Rp: RastPortPtr; mx,my,rx,ry: INTEGER);
(*	:Semantic. Zeichnet eine Ellipse mit Mittelpunkt mx,my und den
	:Semantic. Radien rx und ry.
*)
BEGIN
  Assert(Graphics.AreaEllipse(Rp,mx,my,rx,ry)=0,ADR('Err: AreaEllipse'));
END AreaEllipse;


PROCEDURE AreaCircle(Rp: RastPortPtr; mx,my, r: INTEGER);
(*	:Semantic. Zeichnet einen Kreis mit dem Mittelpunkt mx,my
	:Semantic. und dem Radius r.
*)
BEGIN
  AreaEllipse(Rp,mx,my,r,r)
END AreaCircle;


PROCEDURE AreaEnd(Rp: RastPortPtr);
(*	:Semantic. schliet ein evtl. vorher begonnenes Polygon und startet
	:Semantic. die eigentliche Flloperation.
*)
TYPE
  LongPtr = POINTER TO LONGINT;
  BytePtr = POINTER TO [0..255];
VAR
  i:INTEGER;
  lp: LongPtr;
  bp: BytePtr;
BEGIN
  Assert(Graphics.AreaEnd(Rp)=0,ADR('Err: AreaEnd'));
  (* $R- $V- (Arrriba!) *)
  WITH Rp^.areaInfo^ DO (* Vektoren lschen fr nchstes mal! *)
    lp:=vctrTbl;
    bp:=flagTbl;
    FOR i:=1 TO maxCount DO
      lp^:=0; INC(lp,4); (* clear x,y:INTEGER *)
      bp^:=0; INC(bp); (* clear flags *)
    END;
   (* $R= $V= *)
  END;
END AreaEnd;

END Area.mod
