(*--------------------------------------------------------------------------

:Program.    Topadd.MOD 
:Author.     Uwe Meyer
:Address.    Alex. v. Wacker P.4, 5000 Kln 71 
:Phone,      0221/704915
:Shortcut.   [umk]
:Version.    1.0
:Date.       07.Mar-89
:Copyright.  Public Domain
:Language.   Modula II
:Translator. M2Amiga v3.2d
:Imports.    IntuiStruct, Nicolas Benezan [bne]
:Contents.   Hilfsprozeduren fr Top.MOD

----------------------------------------------------------------------------*)


IMPLEMENTATION MODULE Topadd;


FROM SYSTEM IMPORT
 ADDRESS, ADR, LONGSET;

FROM Arts IMPORT
 Assert;
 
FROM Dos IMPORT
 Lock, UnLock, sharedLock, readOnly, newFile, 
 FileLockPtr, FileInfoBlock, FileInfoBlockPtr, FileHandlePtr,
 Examine, Open, Close, Read, Write, Delay;

FROM Exec IMPORT
 WaitPort, GetMsg, ReplyMsg;

FROM Graphics IMPORT
 Move, Text, RastPortPtr, SetRast, SetAPen, SetDrMd, jam1;

FROM Heap IMPORT
 AllocMem, Deallocate;

FROM Intuition IMPORT
 ScreenPtr, customScreen,
 NewWindow, OpenWindow, CloseWindow, WindowFlags, WindowFlagSet, WindowPtr,
 RefreshWindowFrame, IDCMPFlags, IDCMPFlagSet, IntuiMessagePtr, ModifyIDCMP,
 Gadget, GadgetFlags, GadgetFlagSet, gadgHighbits, strGadget, StringInfo,
 AddGadget, RemoveGadget, RefreshGadgets, ActivationFlags, ActivationFlagSet,
 ActivateGadget, boolGadget, GadgetPtr, Border;
 
FROM IntuiStruct IMPORT
 AllocProc, DeallocProc, StructWindow, StructGadget, StructString,
 StructBorder, BorderEnd, Rectangle, FreeBorder;

FROM Str IMPORT
 Copy, Length, Concat, first, last, noOccur;


PROCEDURE MyRequest (scr : ScreenPtr; title : ADDRESS;
          textlines : INTEGER; VAR text : ARRAY OF reqtext;
          gadwidth : INTEGER; gadtexts : INTEGER; 
          VAR gad : ARRAY OF reqtext) : INTEGER;

CONST	BARGAD		= 34;				(* WinTitel & Gad. *)
	WINDDRAW	= 17;				(* y Zeichenbeginn *)
	FONTWIDTH	= 8;				(* Breite des Font *)
	FONTHEIGHT	= 9;				(* Hhe des Font   *)
	GADHEIGHT	= 11;				(* Gadgethhe      *)
	
VAR	i		: INTEGER;			(* Zhler          *)
	nw		: NewWindow;
	win		: WindowPtr;			(* Requestwindow   *)
	rp		: RastPortPtr;
	width		: INTEGER;			(* Windowbreite    *)
	height		: INTEGER;			(* Windowhhe      *)
	rgad		: ARRAY[0..REQLINES] OF Gadget;	(* Requestergad.   *)
	gadxpos		: INTEGER;			(* xPos des Gad.   *)
	gadoff		: INTEGER;			(* Gadgetabstand   *)
	border		: Border;			(* GadgetBorder    *)
	msg		: IntuiMessagePtr;
	msgclass	: IDCMPFlagSet;
	msgadr		: GadgetPtr;
	gadact		: INTEGER;			(* selekt. Gadget  *)
	len		: INTEGER;			(* akt. Strlnge   *)
	maxlen		: INTEGER;			(* grte Strlnge *)
	y		: INTEGER;			(* akt y Position  *)
		
BEGIN

(*--------------------------------------------------------------------------)
Lngsten Requestertext suchen und Windowgre berechnen
(--------------------------------------------------------------------------*) 

 maxlen := 0;
 FOR i := 0 TO textlines DO
 len := Length (text[i]);
  IF len > maxlen THEN
   maxlen := len;
  END;
 END;
 width := FONTWIDTH * (maxlen + 3);
 height := FONTHEIGHT * (textlines) + BARGAD;

(*-------------------------------------------------------------------------)
Aufbau der Gadgetstruktur und Border
(-------------------------------------------------------------------------*)

 StructBorder (border, 0, 0, 3, jam1, 4, NIL);
  Rectangle (gadwidth, GADHEIGHT);
 BorderEnd;

 gadxpos := 9;
 gadoff := gadwidth + (width - (gadtexts + 1) * gadwidth - 20)
           / gadtexts;
 FOR i := 0 TO gadtexts DO
  StructGadget (rgad[i], gadxpos, -13, gadwidth, GADHEIGHT, 
                GadgetFlagSet {gRelBottom}, ActivationFlagSet {relVerify},
                boolGadget, NIL, NIL, LONGSET {}, i, NIL);
  rgad[i].gadgetRender := ADR (border);
  IF i # gadtexts THEN
   rgad[i].nextGadget := ADR (rgad[i + 1]); 
  INC (gadxpos, gadoff);
  END;
 END;

(*--------------------------------------------------------------------------)
Aufbau der Windowstruktur und ffnung
(--------------------------------------------------------------------------*)

 StructWindow (nw, (640 - width) / 2, (256 - height) / 2, width, height,
               0, 1, IDCMPFlagSet {gadgetUp},
               WindowFlagSet {windowDrag, activate, rmbTrap},
               title, scr, customScreen);
 nw.firstGadget := ADR (rgad[0]);
 win := OpenWindow (nw);
 Assert (win # NIL, ADR ("Error RequestWindow"));
 rp := win^.rPort;
 SetDrMd (rp, jam1);
 SetAPen (rp, 0);
 SetRast (rp, 1);
 RefreshWindowFrame (win);

(*--------------------------------------------------------------------------)
Window mit den Requester- und Gadgettexten beschreiben
(--------------------------------------------------------------------------*)

 y := WINDDRAW;
 FOR i := 0 TO textlines DO
  Move (rp, 10, y);
  Text (rp, ADR (text[i]), Length (text[i]));
  INC (y, FONTHEIGHT);
 END;

 gadxpos := 13;
 FOR i := 0 TO gadtexts DO
  Move (rp, gadxpos, height - 6);
  Text (rp, ADR (gad[i]), Length (gad[i]));
  INC (gadxpos, gadoff);
 END;

(*--------------------------------------------------------------------------)
Auf Aktivierung der Gadgets warten und Gadgetnummer zurckgeben
(--------------------------------------------------------------------------*)

 REPEAT
  WaitPort (win^.userPort);
  msg := GetMsg (win^.userPort);
  IF msg # NIL THEN 
   msgclass := msg^.class;
   msgadr := msg^.iAddress;
   gadact := msgadr^.gadgetID;
   ReplyMsg (msg);
  END;
 UNTIL gadgetUp IN msgclass;  

 FreeBorder (border);
 CloseWindow (win);
 RETURN (gadact);
END MyRequest;


PROCEDURE GetName (VAR win : WindowPtr; VAR name : ARRAY OF CHAR);

CONST	STRINGID	= 998;				(* Strgadgetnummer *)
	GADHEIGHT	= 12;				(* Gadgethhe      *)
	GADWIDTH	= 200;				(* std. Gad.breite *)
	LAST		= -1;				(* Addgad. Status  *)
	MARGIN		= 40;				(* Breite Rand     *)
	
VAR	gad		: Gadget;
	strgad		: StringInfo;
	border		: Border;
	msg		: IntuiMessagePtr;
	msgadr		: GadgetPtr;
	gadact		: INTEGER;			(* selekt. Gadget  *)
	idcmp		: IDCMPFlagSet;			(* IDCMP Retter    *)
	gadwidth	: INTEGER;			(* Gadgetbreite    *)
	xpos		: INTEGER;			(* Gadget x-Pos    *)
	ypos		: INTEGER;			(* Gadget y-Pos    *)
	width		: INTEGER;			(* Windowbreite    *)
	pos		: INTEGER;			(* Pos in Gadliste *)
	ok		: BOOLEAN;			(* Status          *)
	buffer		: ARRAY [0..30] OF CHAR;	(* Stringgad. UNDO *)
	
BEGIN
(*--------------------------------------------------------------------------)
Undobuffer setzen, Stringgadgetgre und Position berechnen
(--------------------------------------------------------------------------*)

 Copy (buffer, name);
 width := win^.width;
 gadwidth := GADWIDTH;
 IF width - MARGIN < GADWIDTH THEN
  gadwidth := width - MARGIN;
 END;
 xpos := (win^.width - gadwidth) / 2;
 ypos := (win^.height - GADHEIGHT) / 2;

(*--------------------------------------------------------------------------)
Gadget- und Stringinfostruktur mit Border erzeugen und an Window anhngen
(--------------------------------------------------------------------------*)

 StructBorder (border, -1, -1, 1, jam1, 4, NIL);
  Rectangle (gadwidth + 2, GADHEIGHT - 2);
 BorderEnd;
 StructString (strgad, name, buffer);
 StructGadget (gad, xpos, ypos, gadwidth, GADHEIGHT, GadgetFlagSet {selected},
               ActivationFlagSet {relVerify, stringCenter},
               strGadget, ADR (border), NIL, LONGSET {}, STRINGID, NIL);
 gad.specialInfo := ADR (strgad);
 gad.nextGadget := NIL;
 pos := AddGadget (win, ADR (gad), LAST);

(*--------------------------------------------------------------------------)
Gadget aktivieren, auf String-RETURN warten und wieder alles entfernen
(--------------------------------------------------------------------------*)

 RefreshGadgets (ADR (gad), win, NIL);
 ok := ActivateGadget (ADR (gad), win, NIL);
 idcmp := win^.idcmpFlags;
 ModifyIDCMP (win, IDCMPFlagSet {gadgetUp});
 REPEAT;
  WaitPort (win^.userPort);
  msg := GetMsg (win^.userPort);
  msgadr := msg^.iAddress;
  gadact := msgadr^.gadgetID;
  ReplyMsg (msg);
 UNTIL gadact = STRINGID;
 pos := RemoveGadget (win, ADR (gad));
 FreeBorder (border);
 ModifyIDCMP (win, idcmp);
END GetName;


PROCEDURE CopyFile (VAR source, dest : ARRAY OF CHAR) : BOOLEAN;

CONST	BUFFER		= 16384;

VAR	mem		: ADDRESS;			(* Speicherzeiger  *)
	len		: LONGINT;			(* Filelnge       *)
	fb		: FileInfoBlockPtr;
	fl		: FileLockPtr;
	fh		: FileHandlePtr;
	ok		: BOOLEAN;			(* Status          *)
		
BEGIN
(*--------------------------------------------------------------------------)
Lnge der Source-Datei ermitteln und in len ablegen. Examinefehler = Abbruch
(--------------------------------------------------------------------------*)

 AllocMem (fb, SIZE (FileInfoBlock), FALSE);
 fl := Lock (ADR (source), sharedLock);
 ok := Examine (fl, fb);
 IF ok THEN
  len := fb^.size;
  UnLock (fl);
  Deallocate (fb);

(*--------------------------------------------------------------------------)
Source File vollstndig in Speicher laden und dann auf Dest kopieren.
Falls FileHandle nicht belegt = Abbruch
(--------------------------------------------------------------------------*)

  AllocMem (mem, len, FALSE);
  Assert (mem # NIL, ADR ("Error Mem CopyFile"));
  fh := Open (ADR (source), readOnly);
  IF fh # NIL THEN
   len := Read (fh, mem, len);
   Close (fh);
   fh := Open (ADR (dest), newFile);
   IF fh # NIL THEN
    len := Write (fh, mem, len);
    Close (fh);
   ELSE
    ok := FALSE;
   END;
  ELSE 
   ok := FALSE;
  END;
  Deallocate (mem);
 END;
 RETURN (ok);
END CopyFile;

END Topadd.
